#Function to perform the main change point distribution calculations with regards to an FMCI approach

#Cversion for speeding up code?

Cver_FINAL.CP_dist.calculate<-function(FMCI_info, krun, X.init){
#Requires FMCI information (contains TP matrices, statespace, )
#krun=length of desired run for a regime to be in progress (this is encorporated in the FMCI_info so we extract from this if necessary and reduce the number of arguments required)
#X.init =  initialisation vector

#Preamble
Z_TPmat_array<-FMCI_info$Z_TPmat #Array containing all the relevant posterior transition matrices
	#3rd dimension represents time, thus there is a posterior TP matrix for each time point
n<-dim(Z_TPmat_array)[3] #Number of time points of interest/
Z_SS<-colnames(Z_TPmat_array[,,1]) #Statespace of the auxiliary MCs under FMCI
Z.star<-length(Z_SS)

Z_absorp_loc<-FMCI_info$Absorp ; Z_contin_loc<-FMCI_info$Contin ; Z_init_loc<-FMCI_info$In #Absorption, Continuation and Initialisation locations

Upsilon.mat<-FMCI_info$Upsilon #Upsilon matrix

max.no.run<-ceiling(n/krun) #The maximum number of change points that could have occurred during the data time period

#Initialisation matrix
#Z.init.mat<-FINAL.CP_init.mat.func(X.init.vec=X.init, Z_init_loc=Z_init_loc, max.no.run=max.no.run, Z_SS=Z_SS)
Z.init.mat<-FINAL.CP_init.mat.func(X.init.vec=X.init, Z_init_loc=Z_contin_loc, max.no.run=max.no.run, Z_SS=Z_SS)

#Psi list/array which stores the probability of being in each state (column), for each auxiliary Markov chain (row) at each time point (depth)
Psi.array<-array(0, dim=c(max.no.run, Z.star, n)) 

#Time 1
Z_temp.prob.mat<-Z.init.mat%*%Z_TPmat_array[,,1] #How do the chain evolve and the corresponding distribution over the states

#Update rows in case we have a reached an absorption state and thus the next chain needs to be activated
	#Second component basically denotes the probability of the previous chain having reached the absorption set
#for(p.row in 2:max.no.run){
#Z_temp.prob.mat[p.row,] <- as.vector(Z_temp.prob.mat[p.row,]  + Z.init.mat[p.row-1,] %*% (Z_TPmat_array[,,1] - diag(Z.star)) %*% Upsilon.mat)
#}

#The C_version
Z_temp.prob.mat<-Psi.Row.update(Psi.mat=Z_temp.prob.mat, Psim1.mat=Z.init.mat, TP.mat=Z_TPmat_array[,,1], Upsilon.mat=Upsilon.mat)

Psi.array[,,1]<-as.matrix(Z_temp.prob.mat)

#Time 2,3, ..., n
for(iter in 2:n){
prev.Psi.mat<-Psi.array[,,iter-1]
Z_temp.prob.mat<-prev.Psi.mat%*% Z_TPmat_array[,,iter] #General evolutions of the chains

#Correction for possibly reaching an absorption state in the previous chain
#for(p.row in 2:max.no.run){
#Z_temp.prob.mat[p.row,] <- as.vector(Z_temp.prob.mat[p.row,]  + prev.Psi.mat[p.row-1,] %*% (Z_TPmat_array[,,iter] - diag(Z.star)) %*% Upsilon.mat)
#}

#The C_version of row updating
Z_temp.prob.mat<-Psi.Row.update(Psi.mat=Z_temp.prob.mat, Psim1.mat=prev.Psi.mat, TP.mat=Z_TPmat_array[,,iter], Upsilon.mat=Upsilon.mat)


Psi.array[,,iter]<-as.matrix(Z_temp.prob.mat)
}

dimnames(Psi.array)<-list(1:max.no.run, Z_SS, 1:n)

#The above lines of code concern the general evolution of the Markov Chains, and the updating mechanism when the previous Markov chains have reached the absorption states,

#Actual CP calculations of interest, based on the general evolution of the MC
CP_calculation_info<-FINAL.CP_calc(Psi.array=Psi.array, TPmat.array=Z_TPmat_array, init.Z.mat=Z.init.mat, krun=krun, Z_absorp_loc=Z_absorp_loc, Z_contin_loc=Z_contin_loc)

list("CP Start Matrix"=CP_calculation_info[[1]], "CP End Matrix"=CP_calculation_info[[2]], "Dist No. CP"=CP_calculation_info[[3]])
}


