#Functions to compute the CP quantities of interest.
#Separating some parts of the this calculation so that it is much easier to request whatever we need.

Cver_Psiarray_calc_Sparse<-function(FMCI_info, krun, X.init){
#Function to compute the Psi array which consists of ndata Psi mats
#where ndata =length of data, Psimat= the matrix which denotes the probability of each chain (corresponds to each correspondence of a run) being in the state for that particular time
#This array is the fundamental quantity in computing the CP quantities of interest
#The function mainly deals with the evolution of the Markov Chains

#Requires FMCI information (contains TP matrices, statespace, )
#krun=length of desired run for a regime to be in progress (this is encorporated in the FMCI_info so we extract from this if necessary and reduce the number of arguments required)
#X.init =  initialisation vector

#Preamble
Z_TPmat_array<-FMCI_info$Z_TPmat #Array containing all the relevant posterior transition matrices
	#3rd dimension represents time, thus there is a posterior TP matrix for each time point
n<-length(Z_TPmat_array)#Number of time points of interest/
Z_SS<-colnames(Z_TPmat_array[[1]]) #Statespace of the auxiliary MCs under FMCI
Z.star<-length(Z_SS)

Z_absorp_loc<-FMCI_info$Absorp ; Z_contin_loc<-FMCI_info$Contin ; Z_init_loc<-FMCI_info$In #Absorption, Continuation and Initialisation locations

Upsilon.mat<-FMCI_info$Upsilon #Upsilon matrix

max.no.run<-ceiling(n/krun) #The maximum number of change points that could have occurred during the data time period

#Initialisation matrix
#Z.init.mat<-FINAL.CP_init.mat.func_Sparse(X.init.vec=X.init, Z_init_loc=Z_init_loc, max.no.run=max.no.run, Z_SS=Z_SS)
Z.init.mat<-FINAL.CP_init.mat.func_Sparse(X.init.vec=X.init, Z_init_loc=Z_contin_loc, max.no.run=max.no.run, Z_SS=Z_SS)

#Psi list/array which stores the probability of being in each state (column), for each auxiliary Markov chain (row) at each time point (depth)
#Psi.array<-array(0, dim=c(max.no.run, Z.star, n))
Psi.array<-vector("list", n)

#Time 1
Z_temp.prob.mat<-Z.init.mat%*%Z_TPmat_array[[1]] #How do the chain evolve and the corresponding distribution over the states

#Update rows in case we have a reached an absorption state and thus the next chain needs to be activated
	#Second component basically denotes the probability of the previous chain having reached the absorption set
#for(p.row in 2:max.no.run){
#Z_temp.prob.mat[p.row,] <- as.vector(Z_temp.prob.mat[p.row,]  + Z.init.mat[p.row-1,] %*% (Z_TPmat_array[[1]] - diag(Z.star)) %*% Upsilon.mat)
#}

#The C_version
Z_temp.prob.mat<-Psi.Row.update_Sparse(Psi.mat=Z_temp.prob.mat, Psim1.mat=Z.init.mat, TP.mat=Z_TPmat_array[[1]], Upsilon.mat=Upsilon.mat)

Psi.array[[1]]<-Z_temp.prob.mat

#Time 2,3, ..., n
for(iter in 2:n){
prev.Psi.mat<-Psi.array[[iter-1]]
Z_temp.prob.mat<-prev.Psi.mat%*% Z_TPmat_array[[iter]] #General evolutions of the chains

#Correction for possibly reaching an absorption state in the previous chain
#for(p.row in 2:max.no.run){
#Z_temp.prob.mat[p.row,] <- as.vector(Z_temp.prob.mat[p.row,]  + prev.Psi.mat[p.row-1,] %*% (Z_TPmat_array[[iter]] - diag(Z.star)) %*% Upsilon.mat)
#}

#The C_version of row updating
Z_temp.prob.mat<-Psi.Row.update_Sparse(Psi.mat=Z_temp.prob.mat, Psim1.mat=prev.Psi.mat, TP.mat=Z_TPmat_array[[iter]], Upsilon.mat=Upsilon.mat)

Psi.array[[iter]]<-Z_temp.prob.mat
}

#dimnames(Psi.array)<-list(1:max.no.run, Z_SS, 1:n)

list("Init Mat"= Z.init.mat, "Psimat Array"= Psi.array)
}

############
CPP_point.calc_Sparse<-function(FMCI_info, krun, Psi_array_info){
#Function which calculations the Prob of ith CP at each time point.
#From this, we can determine the CPP at each time point and the distribution of the number of CP

#Preamble
Z_TPmat_array<-FMCI_info$Z_TPmat #Array containing all the relevant posterior transition matrices
	#3rd dimension represents time, thus there is a posterior TP matrix for each time point
n<-length(Z_TPmat_array) #Number of time points of interest
Z_SS<-colnames(Z_TPmat_array[[1]]) #Statespace of the auxiliary MCs under FMCI
Z.star<-length(Z_SS)

Z_absorp_loc<-FMCI_info$Absorp ; Z_contin_loc<-FMCI_info$Contin ; Z_init_loc<-FMCI_info$In #Absorption, Continuation and Initialisation locations

Upsilon.mat<-FMCI_info$Upsilon #Upsilon matrix

#The Psi mat information
Psi.array<-Psi_array_info$Psi
Z.init.mat<-Psi_array_info$Init

#The actual CP calculation that we want
CP_calculation_info<-FINAL.CP_calc_Sparse(Psi.array=Psi.array, TPmat.array=Z_TPmat_array, init.Z.mat=Z.init.mat, krun=krun, Z_absorp_loc=Z_absorp_loc, Z_contin_loc=Z_contin_loc)

list("CP Start Matrix"=CP_calculation_info[[1]], "CP End Matrix"=CP_calculation_info[[2]], "Dist No. CP"=CP_calculation_info[[3]])
}

####

FINAL.CP_calc_Sparse<-function(Psi.array, TPmat.array, init.Z.mat, krun, Z_absorp_loc, Z_contin_loc){
#Function to convert the distribution of the Markov chain, to the actual CP distributions of interest

Z_SS<-colnames(init.Z.mat); Z.star<-length(Z_SS)
max.no.run<-nrow(init.Z.mat)
#n<-dim(Psi.array)[3]
n<-length(Psi.array)

#UA.vec<-UC.vec<-rep(0, Z.star)
UA.vec<-UC.vec<-Matrix(0, nrow=1, ncol=Z.star)
UA.vec[1,Z_absorp_loc]<-1 # Vector to denote where absorption states are with a 1 in the corresponding place, 0 elsewhere
UC.vec[1,Z_contin_loc]<-1 #Equivalent vector for continuation states

CP_start.mat<-Matrix(0, nrow=n, ncol=max.no.run, sparse=T)
CP_end.mat<-Matrix(0, nrow=n, ncol=max.no.run-1, sparse=T)
	#Matrix which will stored the CP probabilities computing from the distributions of the Markov Chains
	#entry (t, i) denotes the probability of the ith CP occurring at time t

diag.zstar<-Diagonal(Z.star)
for(t.i in 1:(n-1)){
#Run of interest can't happen after the length the underlying statesequence
if(t.i+krun-1<=n){
#Start CP, based on whether the corresponding auxiliary MC will reach the absorption set at time t.i
ifelse({t.i==1} && {krun==1}, CP_start.mat[1,]<-as.vector((Psi.array[[1]] - init.Z.mat)%*%t(UA.vec)), CP_start.mat[t.i, ]<-as.vector((Psi.array[[t.i+krun-1]] - Psi.array[[t.i+krun-2]])%*%t(UA.vec)))
}

#End CP
sp.TP.mat<-TPmat.array[[t.i+1]]
CP_end.mat[t.i, ] <- as.vector(Psi.array[[t.i]][2:max.no.run,] %*% (diag.zstar - sp.TP.mat)%*%t(UC.vec))
#CP_end.mat[t.i, ] <- as.vector(Psi.array[2:max.no.run,,t.i] %*% (diag(Z.star) - TPmat.array[,,t.i+1])%*%UC.vec)
}

dist.No.CP<-colSums(CP_start.mat)
dist.No.CP<--diff(dist.No.CP) #Distribution of 1, 2, ..., max.no.run CPs having occurred

list("Start CP matrix"=CP_start.mat, "End CP matrix"=CP_end.mat, "Dist No. CP"=dist.No.CP)
}


#########
FINAL.CP_init.mat.func_Sparse<-function(X.init.vec, Z_init_loc, max.no.run, Z_SS){
#Function to create the initialisation matrix for use in the CP calculations via FMCI
Z.star<-length(Z_SS)
no.X<-length(X.init.vec)
no.Z.init<-length(Z_init_loc) #Number of initialisation states

Z.init.mat<-Matrix(0, ncol=Z.star, nrow=max.no.run, dimnames=list(1:max.no.run, Z_SS)) #Each row corresponds a different chain (which is run occurrence)
#Each column are the states that each of the auxiliary Markov chains the defined over

run1.init.vec<-rep(0, Z.star)
#Only the first row 1, denoting the progress of the first occurrence and chain contains the stated initialisation vectors.
#This makes sense since we can only consider the mth chain to be in progress is the previous (m-1) are in progress (and have reached their absorption states)

#Strategy is to split the initialisation vector over X, equally among the states in Z which have the corresponding element as it's most recent state
rep.time<-no.Z.init/no.X
run1.init.vec[Z_init_loc]<-rep(X.init.vec/rep.time, each=rep.time)

Z.init.mat[1,]<-run1.init.vec #Add this intialisation configuration to the initialisation vector
Z.init.mat
}
