#Augmenting the FMCI framework so that we need a sustained change for a CP out of regime
#The previous definition of a end of regime is a single change in state in the underlying MC.
#We now try and introduce a sustained definition, so that we allow for one time instances when we are not in the state corresponding to the regime

#The main code that needs to change is where we place the transition probabilities.
	#In particular, for some particular states, returning back to the continuation state is possible.
	
FINAL.FMCI_SS.destination.kstar.reduc<-function(Z.rtuple.vec, Z.rtuple.mat, k, k.star=1, X.SS, symb.vec, ramping.len){
#Function to provide the destination matrix when we observe the new state of the underlying MC
#We now modify the code so we can also need a sustained change for a CP out a regime

#k= sustained time period for CP into a regime
#k.star= sustained time period for CP out of a regime.

r<-ncol(Z.rtuple.mat)-1; no.X<-length(X.SS)
Z.star<-length(Z.rtuple.vec) #Number of states in the augmented space
X.rtuple.mat<-Z.rtuple.mat[,1:r, drop=F]
X.rtuple.vec<-apply(X.rtuple.mat, 1, FUN=paste, collapse=",")
Z.curr.pattern<-Z.rtuple.mat[,r+1]

ref.point<- r-ramping.len #The reference point from which pattern progress are computed from


#Classification of states
Z.cont.loc<-which(Z.rtuple.mat[,r+1]==-1) #Continuation
Z.init.loc<-which(Z.rtuple.mat[,r+1]==-1) #Initialisation
Z.absorp.loc<-which(Z.rtuple.mat[,r+1]==k) #Absorption
Z.transient.loc<-which(Z.rtuple.mat[,r+1]!=k) #Transient

#For symbols of interest, which is all for this particular conference
all.X_cont.tuple<-all.Z_cont.prog<-all.Z_cont.dest<-NULL
for(symb.iter in symb.vec){
#Actual X r-tuples
run.cont.tuple.mat<-cbind(X.rtuple.mat, symb.iter) 
run.cont.tuple<-apply(run.cont.tuple.mat[, -1,drop=FALSE], 1, FUN=paste, collapse=",") #New destination r-tuples
run.cont.tuple[Z.absorp.loc]<-X.rtuple.vec[Z.absorp.loc] #Correct for absorption states, we remain at the same state
all.X_cont.tuple<-c(all.X_cont.tuple, run.cont.tuple)

#New Pattern Progresses
run.Z_cont.prog<-Z.curr.pattern
same.state<-which(run.cont.tuple.mat[,ref.point]==run.cont.tuple.mat[,ref.point+1]) #Those which remain in the same state
same.state.cont<-intersect(same.state, Z.cont.loc) 
#Those where we remain at the corresponding continuation state by seeing the same symbol
diff.state.cont<-setdiff(Z.cont.loc, same.state) #Those where there is a change in state to an interesting state, from continuation state

new.runs<-which(run.cont.tuple.mat[,ref.point]!=run.cont.tuple.mat[,ref.point+1])
#new.runs<-which(match(run.cont.tuple.mat[,ref.point], setdiff(symb.vec, symb.iter), nomatch=0)!=0) 
#Cases when a run in another symbol was in progress but it stops since stopped by an occurrence of another symbol also of interest

run.Z_cont.prog[-c(Z.absorp.loc, same.state.cont, diff.state.cont, new.runs)]<-as.numeric(run.Z_cont.prog[-c(Z.absorp.loc, same.state.cont, diff.state.cont, new.runs)]) + 1
#Those correspond to starting states which are not of interest and we move to corresponding state with the symbol of interest

run.Z_cont.prog[c(new.runs, diff.state.cont)]<-1 #Different symbol of interest, to symbol of interest
run.Z_cont.prog[Z.absorp.loc]<-k #Same progress for absorption states

#Correcting for those where we can return to the continuation states
special.return<-which(Z.curr.pattern<=k.star) #Those below k.star requirement
special.return<-which(0<Z.curr.pattern[special.return]) #Those which aren't continuation or initialisation
special.return<-intersect(special.return, new.runs)
run.Z_cont.prog[special.return]<--1 #For these states, we permit a return to the continuation state, thus saying the regime is still in progress

all.Z_cont.prog<-c(all.Z_cont.prog, run.Z_cont.prog)

#Final Z destination states
Z.run_cont.states<-paste(run.cont.tuple, run.Z_cont.prog, sep="|")
Z.run_cont.dest<-match(Z.run_cont.states, Z.rtuple.vec)
all.Z_cont.dest<-c(all.Z_cont.dest, Z.run_cont.dest)
}

#For symbols not of interest
not.symb.vec<-setdiff(X.SS, symb.vec)
all.X_stop.tuple<-all.Z_stop.prog<-all.Z_stop.dest<-NULL
for(symb.iter in not.symb.vec){

#New X r-tuples
run.stop.tuple.mat<-cbind(X.rtuple.mat, symb.iter) 
run.stop.tuple<-apply(run.stop.tuple.mat[,-1, drop=F], 1, FUN=paste, collapse=",")
run.stop.tuple[Z.absorp.loc]<-X.rtuple.vec[Z.absorp.loc]

#New pattern progress
run.Z_stop.prog<-Z.curr.pattern
run.Z_stop.prog[-c(Z.absorp.loc)]<-0 #All states apart from absorption going a 0 pattern progress state
all.Z_stop.prog<-c(all.Z_stop.prog, run.Z_stop.prog)

#New destinations
Z.run_stop.states<-paste(run.stop.tuple, run.Z_stop.prog, sep="|")
Z.run_stop.dest<-match(Z.run_stop.states, Z.rtuple.vec)
all.Z_stop.dest<-c(all.Z_stop.dest, Z.run_stop.dest)
}

#Correcting for the impossible moves
#all.Z_cont.dest<-all.Z_cont.dest[is.na(all.Z_cont.dest)==FALSE]
#all.Z_stop.dest<-all.Z_cont.dest[is.na(all.Z_stop.dest)==FALSE]

list("Z statespace"=Z.rtuple.vec, "Continue Destin"= all.Z_cont.dest, "End Run Destin"=all.Z_stop.dest)
}

########
FINAL.FMCI_SS.run_kstar.reduc<-function(r.tuples, symb.vec, k, k.star=1, X.SS, ramping.len){
#Statespace for runs in any vector of symbols.
#This thus allows both FMCI statespaces for a specific, generalised, and several symbols to be computed in one function

#Modified SS structure where we can specify 
#k, the sustained period for CP into a regime
#kstar, the sustained period for a CP outof a regime
#Ramping len = reference point wrt to the end from which we count the pattern from (RH to LH) 
	#If equal to 0, this is equivalent to standard FMCI counting


#Preamble
no.X<-length(X.SS); no.Xrtup<-length(r.tuples)
no.symb<-length(symb.vec) #Number of symbols that we are interested in

#r-tuples in matrix form, RHS indicates the most recent, LHS is further into the past
r.tuples.mat<-matrix(as.character(unlist(strsplit(r.tuples, "\\,"))),nrow=no.Xrtup, byrow=T) 
r<-ncol(r.tuples.mat)
ramping.r<-r-ramping.len+1+1

#Initialisation states, j=0
#Z.init<-paste(r.tuples, 0, sep="|") #Initisalisation states for the auxiliary MC chains
#Z.init<-NULL #We might be able to get away with initialising in the continuation states rather than having states specific to initialisation

#Current pattern progress i.e. based on the r-tuples. This includes any absorption states
#Z.curr<-FINAL.FMCI_SS.current.prog(r.tuples=r.tuples, r.tuples.mat=r.tuples.mat, symb=symb.vec, k=k) #Ignore the last column to account ramping and early detection...

#Extra pattern progresses if k>r
#Z.extra<-NULL
#if(k>ramping.r){
#if(no.symb==1){repet.tuple<-paste(rep(symb.vec, ramping.r), collapse=",")}
#else{repet.tuple<-matrix(symb.vec, nrow=no.symb, ncol=r, byrow=F)
#repet.tuple<-apply(repet.tuple, 1, FUN=paste, collapse=",") }
#Z.extra<-paste(repet.tuple, rep((ramping.r+1):k, each=no.symb), sep="|")
#}

#temp.Z_SS<-c(Z.init, Z.curr, Z.extra) #Some of the new statespace
#temp.Z_SS.mat<-FINAL.FMCI_SS.conversion(Z.rtuples=temp.Z_SS, rval=r)[[1]] #Statespace in matrix form. Final column denotes pattern progress of the 

#Building states, as in the first few time steps, we don't go straight from intialisation vectors to the current progress states
#Z.build<-paste(r.tuples, rep((1:r), each=no.Xrtup),sep="|")
#Z.build<-FINAL.FMCI_SS.build.states(temp.Z_SS.mat=temp.Z_SS.mat)

#Continuation states, these are the initialisation states with -1 under the new set up
#Z.cont<-paste(r.tuples, "-1", sep="|")
#true.Z.cont<-FINAL.FMCI_SS.cont.states(temp.Z_SS.mat=temp.Z_SS.mat, k=k)

#A crude go at the Z statespace
Z.SS_Final<-paste(rep(r.tuples, len=(k+2)*no.Xrtup), rep(-1:k, each =no.Xrtup), sep="|")

#Putting it all together and removing any duplications
#Z.SS_Final<-unique(c(Z.cont, Z.init, Z.build, Z.curr,  Z.extra))
Z.SS_Final.mat<-FINAL.FMCI_SS.conversion(Z.rtuples=Z.SS_Final, rval=r)[[1]]
#Into a more intuitive order with the pattern progress variables in ascending order
reorder.vec<-order(as.numeric(Z.SS_Final.mat[,r+1]))
Z.SS_Final<-Z.SS_Final[reorder.vec]
Z.SS_Final.mat<-Z.SS_Final.mat[reorder.vec,]
X.tuple<-apply(Z.SS_Final.mat[,1:r, drop=F], 1, FUN=paste, collapse=",")

#Destinations information
Z.destin.info<-FINAL.FMCI_SS.destination.kstar.reduc(Z.rtuple.vec=Z.SS_Final, Z.rtuple.mat=Z.SS_Final.mat, k=k, k.star=k.star, X.SS=X.SS, symb.vec=symb.vec, ramping.len=ramping.len)
Z.continue.destin<-Z.destin.info$C; Z.end.destin<-Z.destin.info$E

#Classification of states
Z.absorp.loc<-which(Z.SS_Final.mat[,r+1]==k)
Z.cont.loc<-which(Z.SS_Final.mat[,r+1]==-1) 
#Z.cont.loc<-match(true.Z.cont, Z.SS_Final)
Z.init.loc<-which(Z.SS_Final.mat[,r+1]==-1) 
Z.transient.loc<-which(Z.SS_Final.mat[,r+1]!=k) 
list("Z State Space"=Z.SS_Final, "Continue States"=Z.continue.destin, "End States"=  Z.end.destin, "Continuation"=Z.cont.loc, "Initialisation"=Z.init.loc, "Absorption"=Z.absorp.loc, "Transient"=Z.transient.loc, "X-tuples"=X.tuple)
}



######################

FINAL.FMCI_TP.array.reduc<-function(post.TP.array, X.SS, symb.vec, krun, FMCI_SS.info=NULL){
#Function to convert all the posterior transition matrices for underlying MC X
	#into the corresponding TP mats over the FMCI statespace

#Preamble
#post.TP.array = array containing n posterior transition probability matrices, with respect to the n lengthed data sequence
	#Previous versions of this function have had the TPs stored in a list. Arrays are slightly nicer to work with
	#Will consist of 3 dimensions
#X.SS = statespace of the underlying MC X
#symb.vec = vector containing the symbols of which the runs are of interest
#krun = Minimum run length required

Xrtuple.vec<- colnames(post.TP.array[[1]]) #The r-tuple statespace we are working on
n<-length(post.TP.array)
no.symb<-length(symb.vec)

#FMCI Statespace the TPs need to be defined over
if(is.null(FMCI_SS.info)==TRUE)
{FMCI_SS.info<-FINAL.FMCI_SS.run(r.tuples=Xrtuple.vec, symb.vec=symb.vec, k=krun, X.SS=X.SS) }#To compute the statespace for all runs, only perform if the information isn't currently available
Z_SS<-FMCI_SS.info$Z; Z.star<-length(Z_SS) #Statespace
Z_cont_dest<-FMCI_SS.info$Continue; Z_end_dest<-FMCI_SS.info$End #Detination states
Z_cont_loc<-FMCI_SS.info$Continuation; Z_init_loc<-FMCI_SS.info$Init # Location of continuation and initialisation
Z_absorp_loc<-FMCI_SS.info$Absorp; Z_trans_loc<-FMCI_SS.info$Trans #Location of absorption and transient states
X_tuple<-FMCI_SS.info$X

#Basic array template
#Z_TP_array<-array(0, dim=c(Z.star, Z.star, n), dimnames=list(Z_SS, Z_SS, 1:n))
Z_TP_array<-vector("list", n)

#Continue run destination matrix
no.cont.dest<-length(Z_cont_dest); 
cont.dest.mat<-matrix(c(rep(1:Z.star, len=no.cont.dest) , Z_cont_dest), ncol=2, nrow=no.cont.dest, byrow=F)
na.loc<-which(is.na(cont.dest.mat[,2])==TRUE) #Where there are impossible moves
cont.dest.mat<-cont.dest.mat[-na.loc, ]
	#Co-ordinates
cont.dest.tuple.mat<-cbind(X_tuple[cont.dest.mat[,1]], X_tuple[cont.dest.mat[,2] ]) #Corresponding tuple
cont.dest.tuple.reduc<-apply(cont.dest.tuple.mat, 1, FUN=paste, collapse="?") #Start and end tuples in a single element
	
#End run destination matrix
no.end.dest<-length(Z_end_dest)
end.dest.mat<-matrix(c(rep(1:Z.star, len=no.end.dest) , Z_end_dest),ncol=2, nrow=no.end.dest, byrow=F)
	#Co-ordinates
na.loc<-which(is.na(end.dest.mat[,2])==TRUE) #Where there are impossible moves
end.dest.mat<-end.dest.mat[-na.loc, ]
	end.dest.tuple.mat<-cbind(X_tuple[end.dest.mat[,1]], X_tuple[end.dest.mat[,2]]) #Equivalent tuples
end.dest.tuple.reduc<-apply(end.dest.tuple.mat, 1, FUN=paste, collapse="?") #Start and end tuples in a single element
for(iter in 1:n){
curr.X_TP.mat<-post.TP.array[[iter]] #Extract the TP matrix that the new matrix is based upon
curr.X_TP.mat<-as.matrix(curr.X_TP.mat) #Extract the TP matrix that the new matrix is based upon
#dimnames(curr.X_TP.mat)<-list(X.SS, X.SS)
dimnames(curr.X_TP.mat)<-list(Xrtuple.vec,Xrtuple.vec)
#Compute the corresponding transition matrix
Z_FMCI_TP.mat<-FINAL.FMCI_TP.fill.sparse(X_post.TP.mat=curr.X_TP.mat, cont.dest.mat=cont.dest.mat, cont.dest.tuple.mat=cont.dest.tuple.mat, cont.dest.tuple.reduc=cont.dest.tuple.reduc, end.dest.mat=end.dest.mat, end.dest.tuple.mat=end.dest.tuple.mat, end.dest.tuple.reduc=end.dest.tuple.reduc, Z_SS=Z_SS, Z.absorp.loc=Z_absorp_loc)

#Z_TP_array[,,iter]<-Z_FMCI_TP.mat
dimnames(Z_FMCI_TP.mat)<-list(Z_SS, Z_SS)
Z_TP_array[[iter]]<-Z_FMCI_TP.mat
}

#Upsilon matrix calculation for connection between absorption and continuation states
	#These are needed for a new chain to be activated when the previous chain has reached it's absorption set.
#Upsilon.mat<-matrix(0, nrow=Z.star, ncol=Z.star, dimnames=list(Z_SS, Z_SS))
#Upsilon.mat[cbind(Z_absorp_loc, Z_cont_loc)]<-1 #To connect absorption states to the corresponding continuation states
Upsilon.mat<-sparseMatrix(i=Z_absorp_loc, j=Z_cont_loc, x=rep(1, length(Z_absorp_loc)), dimnames=list(Z_SS, Z_SS), dims=c(Z.star, Z.star))

list("Z_TPmat_array"=Z_TP_array, "Absorption"=Z_absorp_loc, "Continuation"=Z_cont_loc, "Inialisation"= Z_init_loc, "Upsilon Matrix"=Upsilon.mat)
}

#######################
FINAL.FMCI_SS.build.states.reduc<-function(temp.Z_SS.mat, ramping.len){
#Function to construct the building states 
#(as from the initialisation states, we cannot go to the current progress states in the first few steps)
#temp.Z_ss.mat = is the matrix form of the states of the auxiliary MC Zs, thus far in the algorithm.
        #Last column is pattern progress, from the ramping length co-ordinate
rval<-ncol(temp.Z_SS.mat)-1
ramp.rval<-rval-ramping.len+1+1
spec.loc.vec<-which(temp.Z_SS.mat[,rval+1]>1) #Which positions do we need to consider and build up to if necessary
build.Z_SS.mat<-NULL
build.Z_SS.vec<-NULL

if(sum(spec.loc.vec)!=0){ #If statement==0, then it means that no position need to have corresponding building states
for(iter in spec.loc.vec){
temp.state<-temp.Z_SS.mat[iter,, drop=F] #Template row to act as a basis
temp.prog<-as.numeric(temp.state[rval+1]) #Extract the pattern progress to work towards
temp.mat<-matrix(temp.state, nrow=temp.prog-1, ncol=rval+1, byrow=T) #Repeat the current tuple by curr.prog-1 for 1,...,curr.prog-1
temp.mat[,rval+1]<-c(1:(temp.prog-1)) #Add the corresponding building states to the final column
build.Z_SS.mat<-rbind(build.Z_SS.mat, temp.mat)
}

build.Z_SS<-FINAL.FMCI_SS.conversion2(Z.rtuples.mat=build.Z_SS.mat)[[1]]
}
else{build.Z_SS<-NULL}
build.Z_SS
}

########################
FINAL.FMCI_TP.fill.sparse<-function(X_post.TP.mat, cont.dest.mat, cont.dest.tuple.mat, cont.dest.tuple.reduc, end.dest.mat, end.dest.tuple.mat, end.dest.tuple.reduc, Z_SS, Z.absorp.loc){
#Function which fills the FMCI transition probability matrix,
#Sparse version
Z.star<-length(Z_SS)
#Z.template<-matrix(0, nrow=Z.star, ncol=Z.star, dimnames=list(Z_SS, Z_SS))
X_SS<-colnames(X_post.TP.mat) #The X statespace

non.zero.prob<-which(X_post.TP.mat!=0, arr.ind=T) #Identify which transition probabilities are 0 and thus we don't need to spend time extracting etc.
non.zero.prob.tuple.table<-cbind(X_SS[non.zero.prob[,1]], X_SS[non.zero.prob[,2]]) #The corresponding tupples
non.zero.prob.tuple.reduc<-apply(non.zero.prob.tuple.table, 1, FUN=paste, collapse="?") #Start and end tuples for those with non-zero entries in a single element

#Entries for progress continuations
cont.non.zero.row_index<-which(match(cont.dest.tuple.reduc, non.zero.prob.tuple.reduc, nomatch=0)!=0) #Non zero entries in continue pattern
cont.run.probs<-X_post.TP.mat[cont.dest.tuple.mat[cont.non.zero.row_index,]] #Pick out the corresponding probabilities
#Z.template[cont.dest.mat[cont.non.zero.row_index,]]<-cont.run.probs #Insert into the appropriate positions for the new TP mat

#Entries for progress halts
end.non.zero.row_index<-which(match(end.dest.tuple.reduc, non.zero.prob.tuple.reduc, nomatch=0)!=0) #Non zero entries in halting pattern
end.run.probs<-X_post.TP.mat[end.dest.tuple.mat[end.non.zero.row_index, ]]
#Z.template[end.dest.mat[end.non.zero.row_index,]]<-end.run.probs

#Z.template[cbind(Z.absorp.loc, Z.absorp.loc)]<-1

#Destination matrix of everything the probabilities of interest
sparse.prob.vec<-c(cont.run.probs, end.run.probs, rep(1, length(Z.absorp.loc)))
sparse.row.destin<-c(cont.dest.mat[cont.non.zero.row_index,1], end.dest.mat[end.non.zero.row_index,1], Z.absorp.loc)
sparse.col.destin<-c(cont.dest.mat[cont.non.zero.row_index,2], end.dest.mat[end.non.zero.row_index,2], Z.absorp.loc)
Z.template<-sparseMatrix(i=sparse.row.destin, j=sparse.col.destin, x=sparse.prob.vec, dims=c(Z.star, Z.star), dimnames=list(Z_SS, Z_SS))
Z.template[cbind(Z.absorp.loc, Z.absorp.loc)]<-1 #Because sparseMatrix makes it non-stochastic for some VERY odd reason.
Z.template
}
