Multi.Dep_WavPer.LogLik<-function(data.mat, theta, init.vec=NULL, order.m, filter.info){

#Main function to compute the log-lik (in C) associated with the the Wavelet Periodogram
#data.mat=wavelet periodogram associated with the time series of interest
#theta=the parameter that we're calculating the log-lik o
#init.vec=initialisation probability vector of underlying MC
#order.m=order(dependency) of the underlying MC
#filter.info=information associated with the wavelet transform (the mother wavelet used)


data.T<-ncol(data.mat); data.J<-nrow(data.mat) #Length of time series and the number of wavelet scales considered.

#Theta decomposition
TP.mat<-theta[[1]] #Base TP matrix
W2.mat<-1/theta[[2]] #Base W2 mat
	#Each row corresponds to different state
	#Each column corresponds to a different scale process

ramping.len<-length(filter.info[[data.J]])
order.m<-2*ramping.len-1

#Some Premable
no.X<-nrow(TP.mat)
if(is.null(init.vec)==TRUE){init.vec<-rep(1, no.X)/no.X}

#A safety guard
if(no.X==1){W2.mat<-matrix(W2.mat, nrow=no.X, ncol=data.J)}

#Extend to the necessary order of the HMM
ext.X_SS.info<-base.to.reduc.TP.mat(base.TP.mat=TP.mat, order.m=order.m) 
ext.TP.mat<-ext.X_SS.info[[1]] #TP matrix on the extended statespace
if(no.X==1){ext.TP.mat[1]<-1} #Because its not working if no.X==1 apparently
	ext.TP.vec<-as.vector(t(ext.TP.mat))
ext.X_SS.mat<-ext.X_SS.info[[2]] #Matrix form of the extended statespace
ext.X_SS<-ext.X_SS.info[[3]] #Vector form of the extended statespace
ext.X_size<-length(ext.X_SS) #Size of the new extended statespace

SigmaD.array<-W2.mat.convert.SigmaD(W2.mat=W2.mat, ext.X_SS.mat=ext.X_SS.mat, filter.info=filter.info)
	#Array containing the state dependent covariance matrices inferred from the state dependent power W2
	#Dim1 and DIm 2= covariance structure between processes for a given state configuration
	#Dim 3= the state configuration of the m-tuple
SigmaD.array_vec<-as.vector(SigmaD.array) #Vectorised version
	
#Initialisation vector on the extended state-space
ext.Xinit.vec<-init.vec[ext.X_SS.mat[,order.m]] #Base it all on the most recent time
ext.Xinit.vec<-ext.Xinit.vec/sum(ext.Xinit.vec) #To make things sum to one

#Alternating sign matrix, required for emission density
alt.sign.label<-paste("Alt", "Sign", "Mat", data.J, sep=".")
if(exists(alt.sign.label)) {alt.sign.mat<-get(alt.sign.label)}
else{alt.sign.mat<-as.matrix(expand.grid(rep(list(c(-1, 1)), data.J)))
assign(alt.sign.label, alt.sign.mat, pos=1)
}

no.combns<-nrow(alt.sign.mat)
alt.sign.vec<-as.vector(t(alt.sign.mat)) #In vector form for the C function

#Data matrix
data.vec<-as.vector(t(data.mat))
dyn.load("Dep_Multi_WavPer_LL.so")
Lik.info<-.C("Dep_Wav_Per_LL", as.double(data.vec), as.integer(data.T), as.double(ext.TP.vec), as.double(SigmaD.array_vec), as.double(ext.Xinit.vec), "li_lo"=double(1), double(data.T), as.integer(ext.X_size), as.integer(data.J), as.double(alt.sign.vec))
#Call the C function to compute the log lik
Lik.info[["li_lo"]] #Return the log-lik associated with the param
}

########################
Multi.Dep_WavPerForward<-function(data.mat, theta, init.vec=NULL, order.m, filter.info){
#Multivariate version of the forward backwards equation for Dependent Wavelet Periodogram processes

#data.mat=matrix version of  theta, init.vec=NULL, order.m, filter.info
#theta=the parameter that we're calculating the log-lik o
#init.vec=initialisation probability vector of underlying MC
#order.m=order(dependency) of the underlying MC
#filter.info=information associated with the wavelet transform (the mother wavelet used)

data.T<-ncol(data.mat); data.J<-nrow(data.mat)

#Theta decomposition
TP.mat<-theta[[1]] #Base TP matrix
W2.mat<-1/theta[[2]] #Base W2 mat
	#Each row corresponds to different state
	#Each column corresponds to a different scale process

ramping.len<-length(filter.info[[data.J]])
order.m<-2*ramping.len-1

#Some Premable
no.X<-nrow(TP.mat)
if(is.null(init.vec)==TRUE){init.vec<-rep(1, no.X)/no.X}

#A safety guard
if(no.X==1){W2.mat<-matrix(W2.mat, nrow=no.X, ncol=data.J)}

#Extend to the necessary order of the HMM
ext.X_SS.info<-base.to.reduc.TP.mat(base.TP.mat=TP.mat, order.m=order.m) 
ext.TP.mat<-ext.X_SS.info[[1]] #TP matrix on the extended statespace
if(no.X==1){ext.TP.mat[1]<-1} #Because its not working if no.X==1 apparently
ext.X_SS.mat<-ext.X_SS.info[[2]] #Matrix form of the extended statespace
ext.X_SS<-ext.X_SS.info[[3]] #Vector form of the extended statespace
ext.X_size<-length(ext.X_SS) #Size of the new extended statespace

SigmaD.array<-W2.mat.convert.SigmaD(W2.mat=W2.mat, ext.X_SS.mat=ext.X_SS.mat, filter.info=filter.info)
	#Array containing the state dependent covariance matrices inferred from the state dependent power W2
	#Dim1 and DIm 2= covariance structure between processes for a given state configuration
	#Dim 3= the state configuration of the m-tuple
	
#Initialisation vector on the extended state-space
ext.Xinit.vec<-init.vec[ext.X_SS.mat[,order.m]] #Base it all on the most recent time
ext.Xinit.vec<-ext.Xinit.vec/sum(ext.Xinit.vec) #To make things sum to one

forward.prob.mat<-t(as.matrix(ext.Xinit.vec))
scaling.vec<-NULL

for(iter in 1:data.T){
emiss.den<-apply(SigmaD.array, FUN=sqMVN.data.density, 3, sq.data.vec=data.mat[,iter])
	#Will compute the emission density with respect to each of the state dependent covariance matrices

alpha.vec<-forward.prob.mat[iter,] %*% ext.TP.mat * emiss.den
Normaliser<-sum(alpha.vec)
scaling.vec<-c(scaling.vec, Normaliser)

forward.prob.mat<-rbind(forward.prob.mat, as.vector(alpha.vec)/Normaliser)
}
loglik<-sum(log(scaling.vec))
list("Forward Prob Mat"=forward.prob.mat[-1,], "Log Lik"=loglik ,"Scaling Constants"=scaling.vec)
}
#########################

#####
Multi.Dep_WavPerBackwards<-function(data.mat, theta, init.vec=NULL, filter.info){

#Multivariate version of the  backwards equation for Dependent Wavelet Periodogram processes
#data.mat=matrix version of  theta, init.vec=NULL, order.m, filter.info
#theta=the parameter that we're calculating the log-lik o
#init.vec=initialisation probability vector of underlying MC
#order.m=order(dependency) of the underlying MC
#filter.info=information associated with the wavelet transform (the mother wavelet used)

data.T<-ncol(data.mat); data.J<-nrow(data.mat)

#Theta decomposition
TP.mat<-theta[[1]] #Base TP matrix
W2.mat<-1/theta[[2]] #Base W2 mat
	#Each row corresponds to different state
	#Each column corresponds to a different scale process

ramping.len<-length(filter.info[[data.J]])
order.m<-2*ramping.len-1

#Some Premable
no.X<-nrow(TP.mat)
if(is.null(init.vec)==TRUE){init.vec<-rep(1, no.X)/no.X}

#A safety guard
if(no.X==1){W2.mat<-matrix(W2.mat, nrow=no.X, ncol=data.J)}

#Extend to the necessary order of the HMM
ext.X_SS.info<-base.to.reduc.TP.mat(base.TP.mat=TP.mat, order.m=order.m) 
ext.TP.mat<-ext.X_SS.info[[1]] #TP matrix on the extended statespace
if(no.X==1){ext.TP.mat[1]<-1} #Because its not working if no.X==1 apparently
ext.X_SS.mat<-ext.X_SS.info[[2]] #Matrix form of the extended statespace
ext.X_SS<-ext.X_SS.info[[3]] #Vector form of the extended statespace
ext.X_size<-length(ext.X_SS) #Size of the new extended statespace

SigmaD.array<-W2.mat.convert.SigmaD(W2.mat=W2.mat, ext.X_SS.mat=ext.X_SS.mat, filter.info=filter.info)
	#Array containing the state dependent covariance matrices inferred from the state dependent power W2
	#Dim1 and DIm 2= covariance structure between processes for a given state configuration
	#Dim 3= the state configuration of the m-tuple
	
#Initialisation vector on the extended state-space
ext.Xinit.vec<-init.vec[ext.X_SS.mat[,order.m]] #Base it all on the most recent time
ext.Xinit.vec<-ext.Xinit.vec/sum(ext.Xinit.vec) #To make things sum to one

backwards.prob.mat<-matrix(0, nrow=data.T+1, ncol=ext.X_size)
backwards.prob.mat[data.T+1, ]<-rep(1, ext.X_size) #Row i+1 corresponds to data point i 
scaling.vec<-NULL

for(iter in (data.T-1):0){ #iter is respect to the data location
emiss.den<-apply(SigmaD.array, FUN=sqMVN.data.density, 3, sq.data.vec=data.mat[,iter+1])
	#Will compute the emission density with respect to each of the state dependent covariance matrices
beta.vec<-ext.TP.mat %*% (backwards.prob.mat[(iter+1)+1,]  * emiss.den)
Normaliser<-sum(beta.vec)
scaling.vec<-c(scaling.vec, Normaliser)

backwards.prob.mat[iter+1,]<-as.vector(beta.vec)/Normaliser
}

list("Backwards Prob Mat"=backwards.prob.mat, "Scaling Vec"=rev(scaling.vec), "SigmaD Array"=SigmaD.array)
}
##########
Multi.Dep_WavPerBackwards_Cver<-function(data.mat, theta, init.vec=NULL, filter.info){

#Multivariate version of the forward backwards equation for Dependent Wavelet Periodogram processes
#Data is now in matrix form I suppose
data.T<-ncol(data.mat); data.J<-nrow(data.mat)

#Theta decomposition
TP.mat<-theta[[1]] #Base TP matrix
W2.mat<-1/theta[[2]] #Base W2 mat
	#Each row corresponds to different state
	#Each column corresponds to a different scale process

ramping.len<-length(filter.info[[data.J]])
order.m<-2*ramping.len-1

#Some Premable
no.X<-nrow(TP.mat)
if(is.null(init.vec)==TRUE){init.vec<-rep(1, no.X)/no.X}

#A safety guard
if(no.X==1){W2.mat<-matrix(W2.mat, nrow=no.X, ncol=data.J)}

#Extend to the necessary order of the HMM
ext.TP.mat<-ext.X_SS.info[[1]] #TP matrix on the extended statespace
ext.X_SS.info<-base.to.reduc.TP.mat(base.TP.mat=TP.mat, order.m=order.m) 
if(no.X==1){ext.TP.mat[1]<-1} #Because its not working if no.X==1 apparently
ext.X_SS.mat<-ext.X_SS.info[[2]] #Matrix form of the extended statespace
ext.X_SS<-ext.X_SS.info[[3]] #Vector form of the extended statespace
ext.X_size<-length(ext.X_SS) #Size of the new extended statespace

SigmaD.array<-W2.mat.convert.SigmaD(W2.mat=W2.mat, ext.X_SS.mat=ext.X_SS.mat, filter.info=filter.info)
	#Array containing the state dependent covariance matrices inferred from the state dependent power W2
	#Dim1 and DIm 2= covariance structure between processes for a given state configuration
	#Dim 3= the state configuration of the m-tuple
SigmaD.array_vec<-as.vector(SigmaD.array) #Vectorised version
	
#Initialisation vector on the extended state-space
ext.Xinit.vec<-init.vec[ext.X_SS.mat[,order.m]] #Base it all on the most recent time
ext.Xinit.vec<-ext.Xinit.vec/sum(ext.Xinit.vec) #To make things sum to one

backwards.prob.mat<-matrix(0, nrow=data.T+1, ncol=ext.X_size)
backwards.prob.mat[data.T+1, ]<-rep(1, ext.X_size) #Row i+1 corresponds to data point i 
scaling.vec<-NULL

#Alternating sign matrix
alt.sign.label<-paste("Alt", "Sign", "Mat", data.J, sep=".")
if(exists(alt.sign.label)) {alt.sign.mat<-get(alt.sign.label)}
else{alt.sign.mat<-as.matrix(expand.grid(rep(list(c(-1, 1)), data.J)))
assign(alt.sign.label, alt.sign.mat, pos=1)
}
no.combns<-nrow(alt.sign.mat)
alt.sign.vec<-as.vector(t(alt.sign.mat)) #In vector form for the C function

#Data matrix
data.vec<-as.vector(t(data.mat))
dyn.load("Dep_Multi_WavPer_LL.so")

back.info<-.C("Dep_WavPer_Back", as.double(data.vec), as.integer(data.T), as.double(t(ext.X_SS.mat)), as.double(SigmaD.array_vec), as.double(ext.Xinit.vec), "Beta_NC"=double(data.T), as.integer(ext.X_size), as.integer(data.J), as.integer(alt.sign.vec), "beta_prob_vec"=double((data.T+1)*ext.X_size))
backwards.prob.mat<-matrix(back.info[["beta_prob_vec"]], nrow=data.T+1, ncol=ext.X_size, byrow=T)
scaling.vec<-back.info[["Beta_NC"]]
list("Backwards Prob Mat"=backwards.prob.mat, "Scaling Vec"=scaling.vec, "SigmaD ARray"=SigmaD.array)
}


#####
Multi.Dep_WavPer.PostTP<-function(data.mat, theta, init.vec=NULL, filter.info){
#Function to compute the posterior transition probability matrices

#data.mat=matrix version of  theta, init.vec=NULL, order.m, filter.info
#theta=the parameter that we're calculating the log-lik o
#init.vec=initialisation probability vector of underlying MC
#filter.info=information associated with the wavelet transform (the mother wavelet used)

#Data is now in matrix form I suppose
data.T<-ncol(data.mat); data.J<-nrow(data.mat)

#Theta decomposition
TP.mat<-theta[[1]] #Base TP matrix
W2.mat<-1/theta[[2]] #Base W2 mat
	#Each row corresponds to different state
	#Each column corresponds to a different scale process
ramping.len<-length(filter.info[[data.J]])
order.m<-2*ramping.len-1
#Some Premable
no.X<-nrow(TP.mat)
if(is.null(init.vec)==TRUE){init.vec<-rep(1, no.X)/no.X}

#A safety guard
if(no.X==1){W2.mat<-matrix(W2.mat, nrow=no.X, ncol=data.J)}

#Extend to the necessary order of the HMM
ext.X_SS.info<-base.to.reduc.TP.mat(base.TP.mat=TP.mat, order.m=order.m) 
ext.TP.mat<-ext.X_SS.info[[1]] #TP matrix on the extended statespace
if(no.X==1){ext.TP.mat[1]<-1} #Because its not working if no.X==1 apparently
ext.X_SS.mat<-ext.X_SS.info[[2]] #Matrix form of the extended statespace
ext.X_SS<-ext.X_SS.info[[3]] #Vector form of the extended statespace
ext.X_size<-length(ext.X_SS) #Size of the new extended statespace

#SigmaD.array<-W2.mat.convert.SigmaD(W2.mat=W2.mat, ext.X_SS.mat=ext.X_SS.mat, filter.info=filter.info)
	#Array containing the state dependent covariance matrices inferred from the state dependent power W2
	#Dim1 and DIm 2= covariance structure between processes for a given state configuration
	#Dim 3= the state configuration of the m-tuple
	
#Initialisation vector on the extended state-space
ext.Xinit.vec<-init.vec[ext.X_SS.mat[,order.m]] #Base it all on the most recent time
ext.Xinit.vec<-ext.Xinit.vec/sum(ext.Xinit.vec) #To make things sum to one
#Backwards equation probabilities

Backwards.Info<-Multi.Dep_WavPerBackwards(data.mat=data.mat, theta=theta, init.vec=init.vec, filter.info)
#Backwards.Info<-Multi.Dep_WavPerBackwards_Cver(data.mat=data.mat, theta=theta, init.vec=init.vec, filter.info)
norm.beta.mat<-Backwards.Info[[1]] #Normalised Backwards probabilities
scaling.vec<-Backwards.Info[[2]] #Scaling constants required if we want to consider the exact backwards probabilities rather than normalised version.
SigmaD.array<-Backwards.Info[[3]]

#Storage
#post.X.TPmat.array<-array(0, dim=c(ext.X_size, ext.X_size, data.T), dimnames=list(ext.X_SS, ext.X_SS, 1:data.T))
post.X.TPmat.array<-vector("list", data.T)


post.X.TP.template<-matrix(0, nrow=ext.X_size, ncol=ext.X_size)# Basic transition matrix template

for(iter in 1:data.T){
#Beta part of the equation
norm.beta_curr<-norm.beta.mat[iter+1,] #The normalised beta probs at the current iteration
norm.beta_prev<-norm.beta.mat[iter,]*scaling.vec[iter] #The normalised beta probs at the previous iteration
beta.part<-outer(norm.beta_curr, norm.beta_prev, FUN="/")
beta.part<-t(beta.part) #So it's in the nice (i,j) configuration

#Emission desnition matrix
emission.density.vec<-apply(SigmaD.array, FUN=sqMVN.data.density, 3, sq.data.vec=data.mat[,iter])
	#Will compute the emission density with respect to each of the state dependent variances/precisions
emission.density.mat<-matrix(emission.density.vec, nrow=ext.X_size, ncol=ext.X_size, byrow=T) #Emission desnity is based on the column which corresponds to the state at time t

#The time dependent posterior transition matrix
post.X.TP.temp<-beta.part*ext.TP.mat*emission.density.mat
dimnames(post.X.TP.temp)<-list(ext.X_SS, ext.X_SS)
#post.X.TPmat.array[,,iter]<-post.X.TP.temp
post.X.TPmat.array[[iter]]<-post.X.TP.temp
}	

post.X.TPmat.array
}
########

W2.mat.convert.SigmaD<-function(W2.mat, ext.X_SS.mat, filter.info){
#Function convert the W2matrix into the set of state-dependent covariance matrices for the emission density
#W2.mat = the matrix containing the state-dependent power
	#Row=State
	#Column=Process
#ext.X_SS.mat = the matrix form of the extendended m-tuple state space
#filter.info = filter information regarding wavelet analysis (mother wavelet)

order.m<-ncol(ext.X_SS.mat); data.J<-ncol(W2.mat)
ext.X_size<-nrow(ext.X_SS.mat)

ext.W2.mat<-apply(ext.X_SS.mat, 1, FUN=extW2.mat.construct, W2.mat=W2.mat)
ext.W2.array<-array(ext.W2.mat, dim=c(data.J, order.m, ext.X_size ))
	#Array containing the state-dependent W2 matrix wrt order of the HMM
	#Dim 1, rows= different scale processes
	#Dim 2, columns= time locations, RHS=more recent
	#Dim 3= the state configuration of the m-tuple

	
#Now compute the correponding state dependent covariance matrices from these extended W2 matrices
SigmaD.array<-apply(ext.W2.array, MAR=3, FUN=SigmaD.func, filter.info=filter.info, no.proc=data.J)
SigmaD.array<-array(SigmaD.array, dim=c(data.J, data.J, ext.X_size))
	#Array containing all the state dependent covariances based on these W2
	#Dim1 and DIm 2= covariance structure between processes for a given state configuration
	#Dim 3= the state configuration of the m-tuple

SigmaD.array #State dependent Sigma D matrices
}

#########
PosteriorDecoding.path<-function(alpha.info, beta.info){
#Function to compute the posterior decoding path given the normalised forwards and backwards info (alpha.info and beta.info respectively)
norm.forwards<-alpha.info[[1]]; forward.scaling.vec<-alpha.info[[3]]
data.T<-nrow(norm.forwards); X.SS<-colnames(norm.forwards)

norm.backwards<-beta.info[[1]];backward.scaling.vec<-beta.info[[2]]
norm.backwards<-norm.backwards[-(data.T+1), ]; backward.scaling.vec<-backward.scaling.vec[-(data.T+1)] #So we have equal length quantities in the forwards and backwards set of equations

#scaling.part<- cumprod(rev(backward.scaling.vec)) /cumprod(rev(forward.scaling.vec))
scaling.part.mat<-matrix(1, nrow=data.T, ncol=ncol(norm.forwards), byrow=T)

smooth.prob.mat<-scaling.part.mat*norm.forwards*norm.backwards #smoothing probability matrix
post.decode.path<-apply(smooth.prob.mat, 1, FUN=which.max)
post.decode.path<-X.SS[post.decode.path]
post.decode.path
}

##############
SigmaD.func<-function (W2.ext.mat, filter.info, no.proc) {
#Function to compute the covariance matrix of the coefficients based on the EWS power
#W2.ext.mat=wavelet power
#filter.info=filter information regarding the wavelet analysis
#no.proc=number of scales involved in wavelet analysis
    j.vec <- c(1:no.proc)
    SigmaD <- matrix(0, nrow = no.proc, ncol = no.proc)
    for (j in 1:no.proc) {
        for (j_p in j:no.proc) {
            SigmaD[j_p, j] <- SigmaD[j, j_p] <- Cov.Dcoeff.func(W2.ext.mat = W2.ext.mat, 
                filter.info = filter.info, j = j, j.prime = j_p)
        }
    }
    SigmaD
}

#######
Cov.Dcoeff.func<-function (j, j.prime, W2.ext.mat, filter.info) {
#Function to compute covariance between two wavelet coefficients at different scales
#j, j.prime= the two scales of interest
#W2.ext.mat=wavelet spectrum
#filter.info=filter information regarding wavelet analysis
    order.m <- ncol(W2.ext.mat)
    no.proc <- nrow(W2.ext.mat)
    largest.mj <- length(filter.info[[no.proc]])
    smallest.mj <- length(filter.info[[1]])
    coeff.j <- filter.info[[j]]
    m.j <- length(coeff.j)
    coeff.j_p <- filter.info[[j.prime]]
    m.j_p <- length(coeff.j_p)
    min.m.j <- min(m.j, m.j_p)
    max.m.j <- max(m.j, m.j_p)
    coeff.convul <- coeff.j[(m.j - min.m.j + 1):m.j] * coeff.j_p[(m.j_p - 
        min.m.j + 1):m.j_p]
    coeff.convul.mat <- outer(coeff.j, coeff.j_p, FUN = "*")
    obs.covar.comp <- NULL
    obs.covar.comp.mat <- matrix(0, nrow = max.m.j, ncol = max.m.j)
    for (lag.obs in 0:(max.m.j - 1)) {
        for (v.lag in 0:(max.m.j - lag.obs - 1)) {
            if (v.lag >= 0) {
                W2.sub <- W2.ext.mat[, (order.m - largest.mj + 
                  1 - (lag.obs)):(order.m - (lag.obs)), drop = F]
                obs.covar <- Cov.obs.func(W2.mat = W2.sub, lag = v.lag, 
                  filter.info, obs.lag = lag.obs)
                obs.covar.comp <- c(obs.covar.comp, obs.covar)
                obs.covar.comp.mat[lag.obs + 1, lag.obs + v.lag + 
                  1] <- obs.covar.comp.mat[lag.obs + v.lag + 
                  1, lag.obs + 1] <- obs.covar
            }
        }
    }
    sum(obs.covar.comp.mat[1:m.j, 1:m.j_p] * coeff.convul.mat)
}

#######
Cov.obs.func<-function (W2.mat, lag = 0, filter.info, obs.lag) {
#Function to compute the covariance between observations under the LSW framework
#W2.mat=the EWS
#lag=the lag between observations
#filter.info=filter information regarding wavelet analysis
    no.proc <- nrow(W2.mat)
    order.m <- ncol(W2.mat)
    max.j <- length(filter.info[[no.proc]])
    covar.estim <- NULL
    for (level in 1:no.proc) {
        filter.coeff <- filter.info[[level]]
        m.j <- length(filter.coeff)
        if (abs(lag) >= m.j) {
            covar.estim <- c(covar.estim, 0)
        }
        else {
            filter.mat <- outer(filter.coeff, filter.coeff, FUN = "*")
            filter.lag <- filter.mat[col(filter.mat) == row(filter.mat) - 
                lag]
            filter.length.lag <- length(filter.lag)
            power <- W2.mat[level, (max.j - m.j + 1):max.j]
            power.vec <- power[1:(m.j - lag)]
            level.cov <- sum(filter.lag * power.vec)
            covar.estim <- c(covar.estim, level.cov)
        }
    }
    sum(covar.estim)
}
