reduc.ext.X.SS<-function(base.X.SS, order.m){
#Function which considers a reduced version of the m-tuples statespace
#We reduce the number of m-tuples by removing those m-tuples which flip between states 

#base.X.SS =the original state space i.e. the symbols that can appear in the state sequence
#order.m = order of the underlying MC

reduc.X.SS<-NULL
reduc.X.SS.mat<-rep("a", order.m) #Dummy row 1
pair.mat<-expand.grid(rep(list(base.X.SS), 2)) #Consider only one possible switch in the m-tuple at the moment.
	#This might need to change if we want to allow for more than one switch

for(row.index in 1:nrow(pair.mat)){
for(repeat.index in 0:order.m){
state<-rep(pair.mat[row.index,], c(repeat.index, order.m-repeat.index)) #To obtain the ramping effect of desire
reduc.X.SS.mat<-rbind(reduc.X.SS.mat, state)
}}

reduc.X.SS.mat<-unique(reduc.X.SS.mat[-1,]) #Remove the dummy m-tuple and consider only the unique m-tuples
reduc.X.SS.vec<-apply(reduc.X.SS.mat, 1, FUN="paste", collapse=",")

reduc.X.SS.mat<-matrix(as.numeric(reduc.X.SS.mat), nrow=nrow(reduc.X.SS.mat), order.m)
dimnames(reduc.X.SS.mat)<-NULL
names(reduc.X.SS.vec)<-NULL
list("Matrix Form"=reduc.X.SS.mat, "Vector Form"=reduc.X.SS.vec)
#Reduced SS in matrix and vector form
}

#############
reduc.ext.X.SS.prob<-function(base.X.SS, order.m, reduc.ext.X.SS, base.X.TP.mat){
#What are the probabilities going to be in this new statespace. 

#i.e. the associated TP with respect to this new reduced SS
#base.X.SS = vector of symbols that can appear in the underlying state sequence
#order.m =order of the underlying MC
#reduc.ext.X.SS=reduced SS information
#base.X.TP.mat=the original, raw TP matrix

X.SS.mat<-as.matrix(reduc.ext.X.SS[[1]]) #expanded SS in matrix form
X.SS.vec<-reduc.ext.X.SS[[2]] #expanded SS in vector form

#What the current progress of runs in the exisiting states
reduc.ext_state_prog<-FINAL.FMCI_SS.current.prog(r.tuples=reduc.ext.X.SS[[2]], r.tuples.mat=reduc.ext.X.SS[[1]], symb=base.X.SS, k=order.m)
reduc.ext_state_prog<-as.numeric(matrix(unlist(strsplit(reduc.ext_state_prog, "\\|")), ncol=2, byrow=T)[,2]) #in nice numerical form

state.int.prob<-which(reduc.ext_state_prog>=order.m) #Those which satisfy the desired sustained period and thus will have interesting transition probabilities
state.boring.prob<-which(reduc.ext_state_prog<order.m) #And those which have a deterministic set up

new.no.X<-length(X.SS.vec)
new.TP.mat<-matrix(0, nrow= length(X.SS.vec), ncol= length(base.X.SS))
new.TP.mat[cbind(state.boring.prob, as.numeric(X.SS.mat[state.boring.prob, order.m]))]<-1 #The boring state transition mechanism is deterministic

new.TP.mat[state.int.prob, ]<- base.X.TP.mat[as.numeric(X.SS.mat[state.int.prob,order.m]), ] #The interesting state transitions
dimnames(new.TP.mat)<-list(X.SS.vec, base.X.SS)
new.TP.mat #The TP matrix with respect to the expanded SS
}

###########
reduc.square.TP.mat<-function(rect.TP.mat, reduc.X.SS.info){
#Function to convert a rectangle TP matrix defined over the reduced SS considered.
	#To a nice square matrix going from m-tuples to m-tuples

#rect.TP.mat=the rectangular m-th order CP
#reduc.X.SS.info=the new expanded, reduced X SS

reduc.X.SS<-rownames(rect.TP.mat) #The reduced SS considered
base.X.SS<-colnames(rect.TP.mat) #The base SS 

no.red.Xsize<-length(reduc.X.SS) #Size of the reduced SS
#temp.sq.TP.mat<-matrix(0, nrow=no.red.Xsize, ncol=no.red.Xsize)
#dimnames(temp.sq.TP.mat)<-list(reduc.X.SS, reduc.X.SS)

#Only considering legitimate moves
non.zero.TP.mat<-rect.TP.mat!=0 #T/F to highlight which are the non-zero probabilities and thus the legitimate moves
non.zero.row<-row(rect.TP.mat)[non.zero.TP.mat] #Row locations of the non-zero probabilities
non.zero.col<-col(rect.TP.mat)[non.zero.TP.mat] #Column locations of the non-zero proababilities
non.zero.prob.vec<-rect.TP.mat[non.zero.TP.mat] #The correponding non-zero probabilities

reduc.X.SS.mat<-as.matrix(reduc.X.SS.info[[1]]) #Matrix representation of the state space
order.m<-ncol(reduc.X.SS.mat)
destin.state.mat<-reduc.X.SS.mat[non.zero.row, 2:order.m, drop=F] #Continuing the previous part of the tuple for states of interest
destin.base.state<-base.X.SS[non.zero.col]#And where they go to for time t
destin.state.mat<-cbind(destin.state.mat, destin.base.state) #The complete product
destin.state.vec<-apply(destin.state.mat, FUN=paste, 1, collapse=",")

#Where is this in the actual statespace of interest
destin.col.coord<-match(destin.state.vec, reduc.X.SS) #This might also come in useful for FMCI stuff

#temp.sq.TP.mat[cbind(non.zero.row, destin.col.coord)]<-non.zero.prob.vec

#If we're being very fancy, then Sparse matrices might come in useful here...for later?
temp.sq.TP.mat<-sparseMatrix(i=non.zero.row, j=destin.col.coord, x=non.zero.prob.vec, dims=c(no.red.Xsize, no.red.Xsize), dimnames=list(reduc.X.SS, reduc.X.SS))
}

############
base.to.reduc.TP.mat<-function(base.TP.mat, order.m){
#Complete function going from base state space and it's transition matrix, to m-tuple reduced SS and it's corresponding square transition matrix

#base.TP.mat=original statespace
#order.m= order of the HMM


base.X.SS<-colnames(base.TP.mat)

#Form the reduced SS
reduc.ext.SS_info<-reduc.ext.X.SS(base.X.SS=base.X.SS, order.m=order.m)

#Form the rectangular TP matrix
rectangle.TP<-reduc.ext.X.SS.prob(base.X.SS=base.X.SS, order.m=order.m, reduc.ext.X.SS=reduc.ext.SS_info, base.X.TP.mat=base.TP.mat)

#Form the equivalend square TP matrix
reduc.sq.TP.mat<-reduc.square.TP.mat(rect.TP.mat=rectangle.TP, reduc.X.SS.info=reduc.ext.SS_info)
list("Reduced Sq TP matrix"=reduc.sq.TP.mat, "R.X_SS.mat"=reduc.ext.SS_info[[1]], "R.X_SS.vec"=reduc.ext.SS_info[[2]])
}

