SMC_CPviaFMCI_Multi.Dep_WavPer<-function(SMC_particles, SMC_weights, krun, k.star, data.mat, X.init.vec=NULL, filter.info){
 #Alternative without the shift in reference where pattern pr
	
	#Perform the CP calculation for numerous model parameters and numerous weights
        #For Multivariate Wavelet Raw Periodogram, accounting for dependency between the processes

#SMC_particles=list of sample parameters from the posterior
#SMC_weights = vector of SMC weights associated with the samples
#krun=sustained length k in underlying statesequence for CP into a regime to have occurred.
#k.star=sustained length k.prime in underlying statesequence for CP out-of-a regime
#data.mat=matrix version of the wavelet periodogram
#X.init.vec=probability initialisation vector of underlying MC
#filter.info = filter information regarding the wavelet transform (mother wavelet)
    
N<-length(SMC_particles) #Number of samples in list
data.T<-ncol(data.mat); data.J<-nrow(data.mat) #Length of time series and the number of wavelet scales included in the analysis
max.no.CP<-ceiling(data.T/krun) #Maximum number of possible CP that could have occurred

no.X<-nrow(SMC_particles[[1]][[1]]) #Number of states assumed in HMM
X.SS<-1:no.X #Statespace of HMM

if(is.null(X.init.vec)==TRUE){X.init.vec<-rep(1, no.X)/no.X} #If no probability initialisation vector is provided

Gen_CPStart.mat<-matrix(0, nrow=data.T, ncol=max.no.CP) #CPP mat for the CP starts (starts of regimes)
Gen_CPEnd.mat<-matrix(0, nrow=data.T, ncol=max.no.CP-1) #CPP mat for the CP ends (end of regimes)
Gen_NumCP.vec<-rep(0, len=max.no.CP-1) #Distribution of number of CPs

ramping.len<-length(filter.info[[data.J]]) #Contamination due to overlapping nature of NDWT
order.m<-2*ramping.len-1 #Order (Dependency) induced in HMM
cat(order.m)
for(iter in 1:N){
#Extract a model parameter setting and weights
param.settings<-SMC_particles[[iter]]
param.weight<-SMC_weights[iter]

#Corresponding Posterior transition probability matrices associated with parameter set
post.tp.array<-Multi.Dep_WavPer.PostTP(data.mat=data.mat, theta=param.settings, init.vec=X.init.vec, filter.info=filter.info)

#no.X<-nrow(post.tp.array[,,1])
#Single computation of the FMCI state space rather than needlessly repeating for each parameter. It stays the same for each parameter sample
if(iter==1){
#Something a lot more sophisticated than this could be done. Something which actually uses the base, original state space
ext.X_SS<-colnames(post.tp.array[[1]]) #The extended statespace
ext.X_size<-length(ext.X_SS)
ext.X_SS.mat<-matrix(as.numeric(unlist(strsplit(ext.X_SS, "\\,"))), nrow=ext.X_size, byrow=T) #matrix version of the extended state space
order.m<-ncol(ext.X_SS.mat)

#What the current progress of runs in the exisiting states
ext_SS_prog<-FINAL.FMCI_SS.current.prog(r.tuples=ext.X_SS, r.tuples.mat=ext.X_SS.mat, symb=X.SS, k=order.m)
ext_SS_prog<-as.numeric(matrix(unlist(strsplit(ext_SS_prog, "\\|")), ncol=2, byrow=T)[,2]) #in nice numerical form
pure.states<-which(ext_SS_prog==order.m) #Which are the pure states of interest which correspond to our CPs

#relabel.X_SS<-1:length(ext.X_SS)
#FMCI_SS.info<-FINAL.FMCI_SS.run_kstar(r.tuples=as.character(relabel.X_SS), symb.vec=pure.states, k=krun, k.star=k.star, X.SS=relabel.X_SS)
#The necessary FMCI SS information corresponding to this set up
#colnames(post.tp.array)<-relabel.X_SS; rownames(post.tp.array)<-relabel.X_SS

#Initialisation vector on the extended state-space
#ext.Xinit.vec<-X.init.vec[ext.X_SS.mat[,order.m]] #Base it all on the most recent time
#ext.Xinit.vec<-ext.Xinit.vec/sum(ext.Xinit.vec) #To make things sum to one

#Compute the FMCI State space based on progress on tuples a etc..
FMCI_SS.info<-FINAL.FMCI_SS.run_kstar.reduc(r.tuples=ext.X_SS, symb.vec=X.SS, k=krun, k.star=k.star, X.SS=X.SS, ramping.len=ramping.len)
ext.Xinit.vec<-X.init.vec
}

#Corresponding FMCI transition information
#colnames(post.tp.array)<-relabel.X_SS; rownames(post.tp.array)<-relabel.X_SS
#FMCI.info<-FINAL.FMCI_TP.array(post.TP.array=post.tp.array, X.SS=relabel.X_SS, symb.vec=pure.states, krun=krun, FMCI_SS.info=FMCI_SS.info)
FMCI.info<-FINAL.FMCI_TP.array.reduc(post.TP.array=post.tp.array, X.SS=X.SS, symb.vec=X.SS, krun=krun, FMCI_SS.info=FMCI_SS.info)

#Evoltuion of the Markov Chains in initialisation probability matrices
#Psi.array.info<-Cver_Psiarray_calc(FMCI_info=FMCI.info, krun=krun, X.init=ext.Xinit.vec)
Psi.array.info<-Cver_Psiarray_calc_Sparse(FMCI_info=FMCI.info, krun=krun, X.init=ext.Xinit.vec)

#Corresponding CPP distribution
#CP_info<-CPP_point.calc(FMCI_info=FMCI.info, krun=krun, Psi_array_info=Psi.array.info) #CPP at point in time and distribution of number of CPs
CP_info<-CPP_point.calc_Sparse(FMCI_info=FMCI.info, krun=krun, Psi_array_info=Psi.array.info) #CPP at point in time and distribution of number of CPs

#Multiply by weights and add to generalised CP characterisitic distributions
Gen_CPStart.mat<-Gen_CPStart.mat+param.weight*CP_info[[1]]
Gen_CPEnd.mat<-Gen_CPEnd.mat+param.weight*CP_info[[2]]
Gen_NumCP.vec<-Gen_NumCP.vec+param.weight*CP_info[[3]]
}

ramp.len<-length(filter.info[[data.J]])

#Shifting due to the ramping induced by the NDWT
Shift.Gen_CPStart.mat<-Gen_CPStart.mat
#Shift.Gen_CPStart.mat<-Matrix(0, nrow=data.T, ncol=max.no.CP)

Shift.Gen_CPEnd.mat<-Gen_CPEnd.mat
#Shift.Gen_CPEnd.mat<-Matrix(0, nrow=data.T, ncol=max.no.CP-1)

#Shift.Gen_CPStart.mat[(ramp.len+1):data.T,]<-Gen_CPStart.mat[1:(data.T-ramp.len), ]
#Shift.Gen_CPEnd.mat[(ramp.len+1):data.T,]<-Gen_CPEnd.mat[1:(data.T-ramp.len), ]


list("Generalised CPP Start"=Shift.Gen_CPStart.mat, "Generalised CPP End"=Shift.Gen_CPEnd.mat, "Generalised No. CP Dist"=Gen_NumCP.vec)
#"Generalised CPP Start"=Shift.Gen_CPStart.mat=CPP into a regime
#"Generalised CPP End"=Shift.Gen_CPEnd.mat=CPP out of a regime
#"Generalised No. CP Dist"=Gen_NumCP.vec=Distribution of number of CP
}


##########################
SMC_CPviaFMCI_Multi.Dep_WavPer_Alt<-function(SMC_particles, SMC_weights, krun, k.star, data.mat, X.init.vec=NULL, filter.info){
 #Alternative without the shift in reference where pattern pr
	
	#Perform the CP calculation for numerous model parameters and numerous weights
        #For Multivariate Wavelet Raw Periodogram, accounting for dependency between the processes
        
#SMC_particles=list of sample parameters from the posterior
#SMC_weights = vector of SMC weights associated with the samples
#krun=sustained length k in underlying statesequence for CP into a regime to have occurred.
#k.star=sustained length k.prime in underlying statesequence for CP out-of-a regime
#data.mat=matrix version of the wavelet periodogram
#X.init.vec=probability initialisation vector of underlying MC
#filter.info = filter information regarding the wavelet transform (mother wavelet)

    
N<-length(SMC_particles) #Number of particles
data.T<-ncol(data.mat); data.J<-nrow(data.mat)
max.no.CP<-ceiling(data.T/krun) #Maximum number of possible CP

no.X<-nrow(SMC_particles[[1]][[1]]) #Number of states under consideration 
X.SS<-1:no.X

if(is.null(X.init.vec)==TRUE){X.init.vec<-rep(1, no.X)/no.X}

Gen_CPStart.mat<-matrix(0, nrow=data.T, ncol=max.no.CP)
Gen_CPEnd.mat<-matrix(0, nrow=data.T, ncol=max.no.CP-1)
Gen_NumCP.vec<-rep(0, len=max.no.CP-1)

ramping.len<-length(filter.info[[data.J]])
order.m<-2*ramping.len-1
cat(order.m)
for(iter in 1:N){
#Extract a model parameter setting
param.settings<-SMC_particles[[iter]]
param.weight<-SMC_weights[iter]

#Correspond Posterior transition probability matrices associated with parameter
post.tp.array<-Multi.Dep_WavPer.PostTP(data.mat=data.mat, theta=param.settings, init.vec=X.init.vec, filter.info=filter.info)

#no.X<-nrow(post.tp.array[,,1])
#Single computation of the FMCI SS rather than computing it for each parameter. It remains the same for each parameter sample
if(iter==1){
#Something a lot more sophisticated than this could be done. Something which actually uses the base, original state space
ext.X_SS<-colnames(post.tp.array[[1]]) #The extended statespace
ext.X_size<-length(ext.X_SS)
ext.X_SS.mat<-matrix(as.numeric(unlist(strsplit(ext.X_SS, "\\,"))), nrow=ext.X_size, byrow=T) #matrix version of the extended state space
order.m<-ncol(ext.X_SS.mat)

#What the current progress of runs in the exisiting states
ext_SS_prog<-FINAL.FMCI_SS.current.prog(r.tuples=ext.X_SS, r.tuples.mat=ext.X_SS.mat, symb=X.SS, k=order.m)
ext_SS_prog<-as.numeric(matrix(unlist(strsplit(ext_SS_prog, "\\|")), ncol=2, byrow=T)[,2]) #in nice numerical form
pure.states<-which(ext_SS_prog==order.m) #Which are the pure states of interest which correspond to our CPs

#relabel.X_SS<-1:length(ext.X_SS)
FMCI_SS.info<-FINAL.FMCI_SS.run_kstar(r.tuples=as.character(relabel.X_SS), symb.vec=pure.states, k=krun, k.star=k.star, X.SS=relabel.X_SS)
#The necessary FMCI SS information corresponding to this set up
colnames(post.tp.array)<-relabel.X_SS; rownames(post.tp.array)<-relabel.X_SS

#Initialisation vector on the extended state-space
#ext.Xinit.vec<-X.init.vec[ext.X_SS.mat[,order.m]] #Base it all on the most recent time
#ext.Xinit.vec<-ext.Xinit.vec/sum(ext.Xinit.vec) #To make things sum to one


#The old fashion way which will fail no doubt, with a rampling length included
FMCI_SS.info<-FINAL.FMCI_SS.run_kstar.reduc(r.tuples=ext.X_SS, symb.vec=X.SS, k=krun, k.star=k.star, X.SS=X.SS, ramping.len=ramping.len)
ext.Xinit.vec<-X.init.vec
}

#Corresponding FMCI transition information
#colnames(post.tp.array)<-relabel.X_SS; rownames(post.tp.array)<-relabel.X_SS
#FMCI.info<-FINAL.FMCI_TP.array(post.TP.array=post.tp.array, X.SS=relabel.X_SS, symb.vec=pure.states, krun=krun, FMCI_SS.info=FMCI_SS.info)
FMCI.info<-FINAL.FMCI_TP.array.reduc(post.TP.array=post.tp.array, X.SS=X.SS, symb.vec=X.SS, krun=krun, FMCI_SS.info=FMCI_SS.info)

#Evoltuion of the Markov Chains via initialisation probability matrices
#Psi.array.info<-Cver_Psiarray_calc(FMCI_info=FMCI.info, krun=krun, X.init=ext.Xinit.vec)
Psi.array.info<-Cver_Psiarray_calc_Sparse(FMCI_info=FMCI.info, krun=krun, X.init=ext.Xinit.vec)

#Corresponding CPP distribution
#CP_info<-CPP_point.calc(FMCI_info=FMCI.info, krun=krun, Psi_array_info=Psi.array.info) #CPP at point in time and distribution of number of CPs
CP_info<-CPP_point.calc_Sparse(FMCI_info=FMCI.info, krun=krun, Psi_array_info=Psi.array.info) #CPP at point in time and distribution of number of CPs

#Multiply by weights and add to generalised CP characterisitic distributions
Gen_CPStart.mat<-Gen_CPStart.mat+param.weight*CP_info[[1]] #CPP into a regime
Gen_CPEnd.mat<-Gen_CPEnd.mat+param.weight*CP_info[[2]] #CPP out of a regime
Gen_NumCP.vec<-Gen_NumCP.vec+param.weight*CP_info[[3]] #Distribution of number of CPs
}

ramp.len<-length(filter.info[[data.J]])

#Shifting due to the ramping induced by the NDWT
Shift.Gen_CPStart.mat<-Gen_CPStart.mat
#Shift.Gen_CPStart.mat<-Matrix(0, nrow=data.T, ncol=max.no.CP)

Shift.Gen_CPEnd.mat<-Gen_CPEnd.mat
#Shift.Gen_CPEnd.mat<-Matrix(0, nrow=data.T, ncol=max.no.CP-1)

#Shift.Gen_CPStart.mat[(ramp.len+1):data.T,]<-Gen_CPStart.mat[1:(data.T-ramp.len), ]
#Shift.Gen_CPEnd.mat[(ramp.len+1):data.T,]<-Gen_CPEnd.mat[1:(data.T-ramp.len), ]


list("Generalised CPP Start"=Shift.Gen_CPStart.mat, "Generalised CPP End"=Shift.Gen_CPEnd.mat, "Generalised No. CP Dist"=Gen_NumCP.vec)
#"Generalised CPP Start"=Shift.Gen_CPStart.mat=CPP into a regime
#"Generalised CPP End"=Shift.Gen_CPEnd.mat=CPP out of a regime
#"Generalised No. CP Dist"=Gen_NumCP.vec=Distribution of number of CP

}

