#More supplementary functions

dirchlet.sample<-function(shape.vec, no.X=2, no.samp=10){
#Function to provide a sample from the Dirichlet distribution 
	#To sample from this distribution, we sample independently from some gammas
	#And normalise by their sum

#shape.vec=specified strength associated with dirichlet
#no.X= number of states assumed in HMM
#no.samp= number of samples we want to obtain from dirichlet distribution

dir.sample.mat<-matrix(1, ncol=1, nrow=no.X)
iter<-1
while(iter<=no.samp){
gamma.samples<-rgamma(n=no.X, shape=shape.vec, scale=1)
NC<-sum(gamma.samples)
dir.samples<-gamma.samples/NC
dir.sample.mat<-cbind(dir.sample.mat, dir.samples)
iter<-iter+1
}
dir.sample.mat[,-1] #Matrix containing no.sampl samples from the corresponding dirichlet. Each column represents a different sample

#Output, no.samp from a dirichlet distribution
}

######
dirchlet.log.density<-function(xvec, shape.vec){
#Function to provide the log-density for a given sample from a dirichlet distribution
#x.vec=sample
#shape.vec=corresponding shape vector

part1<-sum((shape.vec-1)*log(xvec))
lB.NC<-sum(lgamma(shape.vec)) - lgamma(sum(shape.vec))
l.density<-part1 - lB.NC
l.density
#density according dirichlet distribution
}

######
TP.mat.construct<-function(tp.vec){
#Given a probability vector containing p11 and p21, construct the corresponding TP mat
#tp.vec=vector containing TP probs excluding the alst column. Assumed to follow p1., p2......format i.e. row TP vectors

no.tprobs<-length(tp.vec) #Number of transition probability vectors that we have available
no.X<-(1+ sqrt(1+4*no.tprobs))/2 #So the number of underlying states based on this
TP.mat<-matrix(tp.vec, nrow=no.X, ncol=no.X-1, byrow=T) #Place available tp's in matrix
TP.mat<-cbind(TP.mat, 1-rowSums(TP.mat)) #Compute the remaining column of tp by row sum constraint
dimnames(TP.mat)<-list(1:no.X, 1:no.X)
TP.mat #Return the corresponding square TP matrix
}

###

tprob.proposals<-function(curr.tprob, prop.var, corr.tp=-0.75, time){
#Function to perform proposals for transition probabilities jointly 
    #Only 2-state proposals only at the moment

#curr.tprob=the current TP in vector form
#prop.var=the proposal variance utilised
#corr.tp=the proposal correlation induced
#time=the SMC sampler iteration

curr.logits<-log(curr.tprob/(1-curr.tprob)) #Logits
Sigma<-diag(2) #Create the covariance matrix
diag(Sigma)<-prop.var
Sigma[1,2]<-Sigma[2,1]<-corr.tp #Place in the covariance

prop.logits<-curr.logits+rmvnorm(1, mean=rep(0,2), Sigma/(time-1)) #Proposal logits
tprob.prop<-exp(prop.logits)/(1+exp(prop.logits))
tprob.prop #Return proposal logits
}

####

TPmat.proposal.logit<-function(curr.TPmat, prop.var){
#Function to perform TP matrix proposals when there are more than two states, logit based system

#curr.TPmat=the current TP matrix that we want to mutate
#prop.var=the proposal variance utilised

#We perform at the moment, a independent proposal structure (that is proposals on the logits are independent of each other/

no.X<-nrow(curr.TPmat) #Number of states under consideration
prop.TPmat<-curr.TPmat

for(row in 1:no.X){ #Mutate each row separately
curr.tp.row<-curr.TPmat[row,] #The transition probability vector row
curr.logit<-log(curr.tp.row/curr.tp.row[no.X])  #The corresponding logits 

#Proposal
prop.logit<-curr.logit+rnorm(no.X, mean=0, sd=sqrt(prop.var)) #Proposal logits based on RWMH
prop.logit[no.X]<-0 #The last logit needs to remain the same
prop.tp.row<-exp(prop.logit)/sum(exp(prop.logit)) #Now the corresponding proposal transition probability row vector

prop.TPmat[row,]<-prop.tp.row
}

prop.TPmat #The proposal TP matrix
}

####################

TPmat.proposal.weights<-function(curr.TPmat, prop.var){
#Function to perform TP matrix proposals when there are more than two states, based on a weights system (as opposed to logit system)

#curr.TPmat = the current TP matrix
#prop.var = the proposal variance used.
#We perform at the moment, a independent proposal structure (that is proposals on the logits are independent of each other/

no.X<-nrow(curr.TPmat) #Number of states under consideration
prop.TPmat<-curr.TPmat

for(row in 1:no.X){
curr.tp.row<-curr.TPmat[row,] #The transition probability vector row
curr.weights<-curr.tp.row #The current weights

#Proposals on the weights
prop.weights<-exp(log(curr.weights)+rnorm(no.X, mean=0, sd=sqrt(prop.var))) #Proposal of weights on log scale (as opposed to a logit scale)
prop.tp.row<-prop.weights/sum(prop.weights) #Now the corresponding proposal transition probability row vector

prop.TPmat[row,]<-prop.tp.row
}

prop.TPmat #The proposal TP matrix
}

##################
weights.to.TPmat<-function(weight.mat){
#Function to go from a weight matrix (each row representing a different unormalised weight vector) to a transition probability vector
	#This thus normalises the weights as necessary
	
NC.vec<-rowSums(weight.mat) #Normalising constants for the vectors
TP.mat<-weight.mat/NC.vec
TP.mat
}

###################

System_Resample<-function(index.vec, w.vec){
#Function to perform systematic resampling at some iteration
#index.vec =  vector from 1 to number of particles. Acting as labels for the particles
#w.vec =  vector of weights for the associated particles

N<-length(w.vec) #Number of particles to consider
N.vec<-rep(0, N)

U1<-runif(1, min=0, max=1/N)
U_i.vec<-U1+((index.vec[-1]-1)/N) #for i=2, ...N
U.vec<-c(U1, U_i.vec)
CuW.vec<-cumsum(w.vec)

N.vec[1]<-sum(U.vec<=CuW.vec[1])

for(i in 2:N){
U.vec.upd<-U.vec[U.vec<=CuW.vec[i] ]
N.vec[i]<-sum(CuW.vec[i-1]<=U.vec.upd) #Number of elements in the set of interest
}
res.index<-rep(index.vec, times=N.vec)
return(res.index) #Returns the indices of the particles that we should resample
}

#######

SMC_PAC.proposal<-function(curr.PAC.vec, prop.PAC.var){
#Function to provide proposal PAC values given current values

rval<-length(curr.PAC.vec)
if(curr.PAC.vec[1]!=0){
curr.logit<-log((curr.PAC.vec+1)/(1-curr.PAC.vec)) #The corresponding logit vector when the PAC are transformed onto the (0,1) interval

prop.logit<-curr.logit + rnorm(rval, mean=0, sd= sqrt(prop.PAC.var))
prop.PAC.vec<-2*(exp(prop.logit)/(exp(prop.logit)+1)) -1
}
else{prop.PAC.vec<-curr.PAC.vec} #If the AR1==0 exactly, then retain the setting of Ar1==0
prop.PAC.vec
}

########
hyper.vec.sep<-function (hyper.vec, no.X = 2) 
{
    tp.hyper <- hyper.vec[1]
    mu.hyper <- hyper.vec[2:3]
    prec.hyper <- hyper.vec[4:5]
    list(TP = tp.hyper, MU = mu.hyper, PREC = prec.hyper)
}
