Sim.Data.GaussMM.Orderm<-function(X.UndMC, order.m, mu.vec,sd.vec){
#Function Simulating data according to a Gaussian Markov Mixture 
#X.UndMC=vector of the path of the underlying MC that we simulate according to
#order.m=order of underlying MC in terms of how far back in terms of states
#mu.vec=state dependent emission mean vector
#sd.vec=state dependent emission sd vecto

X.UndMC<-as.numeric(X.UndMC)
no.X<-length(mu.vec) #Number of underlying states
data.vec<-NULL
for(time in 1:length(X.UndMC)){
eff.window.len<-min(order.m, time) 
    #Effective window size since for 1,...m-1, it won't work (efficiently number of previous states)
int.states<-X.UndMC[time:(time-eff.window.len+1)]
    #Actual states of interest from given state sequence
eff.mu<-mean(mu.vec[int.states]) #How to summarise the last (m) means
eff.sd<-mean(sd.vec[int.states]) #How to summarise the last (m) sds
data.vec<-c(data.vec, rnorm(1, mean=eff.mu, sd=eff.sd))
    #Gaussian distributed data with respect to the underyling states (the parameters are underlying state dependent)
}

data.vec #Gaussian Markov Mixutre data
}


####
Sim.Data.Hamil.MSAR<-function(X.UndMC, mu.vec, sd.vec, AR.vec){
#Function to simulate data according to a Hamilton Markov Switching Autoregressive model
    #We allow for the mean and variance to switch according to the underlying MC
    #The Autoregressive parameters remain the same no matter what the underlying state is.

#X.UndMC =  Underlying state sequence of MC that we sample according to
#mu.vec= Vector of state dependent means
#mu.vec =Vector of state dependent sd
#AR.vec= Vector of AR parameter, recent to past from left to right

X.UndMC<-as.numeric(X.UndMC)
no.X<-length(mu.vec)
data.T<-length(X.UndMC)
order.r<-length(AR.vec) #AR dependency

data.vec<-NULL
for(time in 1:data.T){
int.time.points<- unique(max(1,time-order.r):time) #Time points of interest, the first argument is necessary since in the cases when time<r, there aren't enough observations to look back onto
    #The unique is for the case of time=1, when we don't want to account for time point 1 twice.
int.states<-X.UndMC[int.time.points] #The corresponding underlying states at these time locations
temp.orderr<-length(int.states) - 1

if(temp.orderr==0){eff.mu.vec<-mu.vec[int.states]} #For the case when there is nothing to look back on at all
else{eff.mu.vec<-data.vec[int.time.points[-(temp.orderr+1)]]- mu.vec[int.states[-(temp.orderr+1)]] 
eff.mu.vec<-sum(eff.mu.vec * rev(AR.vec[1:temp.orderr])) + mu.vec[int.states[temp.orderr+1]] #The effective means, which is of the form AR.param * (data - und.mean)
    #The AR.vec is reverse so we have AR1 corresponding to y_{t-1} etc.
}
eff.sd<-sd.vec[X.UndMC[time]] #The underlying std dev at the current time point we are at
data.vec<-c(data.vec, rnorm(1, mean=eff.mu.vec, sd=eff.sd))
}
data.vec #Simulated HMSAR data
}


#######

reduc.ext.X.SS<-function(base.X.SS, order.m){
#Function which considers a reduced version of the m-tuples statespace
#We reduce the number of m-tuples by removing those m-tuples which flip between states 

#base.X.SS =the original state space i.e. the symbols that can appear in the state sequence
#order.m = order of the underlying MC

reduc.X.SS<-NULL
reduc.X.SS.mat<-rep("a", order.m) #Dummy row 1
pair.mat<-expand.grid(rep(list(base.X.SS), 2)) #Consider only one possible switch in the m-tuple at the moment.
	#This might need to change if we want to allow for more than one switch

for(row.index in 1:nrow(pair.mat)){
for(repeat.index in 0:order.m){
state<-rep(pair.mat[row.index,], c(repeat.index, order.m-repeat.index)) #To obtain the ramping effect of desire
reduc.X.SS.mat<-rbind(reduc.X.SS.mat, state)
}}

reduc.X.SS.mat<-unique(reduc.X.SS.mat[-1,]) #Remove the dummy m-tuple and consider only the unique m-tuples
reduc.X.SS.vec<-apply(reduc.X.SS.mat, 1, FUN="paste", collapse=",")

reduc.X.SS.mat<-matrix(as.numeric(reduc.X.SS.mat), nrow=nrow(reduc.X.SS.mat), order.m)
dimnames(reduc.X.SS.mat)<-NULL
names(reduc.X.SS.vec)<-NULL
list("Matrix Form"=reduc.X.SS.mat, "Vector Form"=reduc.X.SS.vec)
#Reduced SS in matrix and vector form
}

#############
reduc.ext.X.SS.prob<-function(base.X.SS, order.m, reduc.ext.X.SS, base.X.TP.mat){
#What are the probabilities going to be in this new statespace. 

#i.e. the associated TP with respect to this new reduced SS
#base.X.SS = vector of symbols that can appear in the underlying state sequence
#order.m =order of the underlying MC
#reduc.ext.X.SS=reduced SS information
#base.X.TP.mat=the original, raw TP matrix

X.SS.mat<-as.matrix(reduc.ext.X.SS[[1]]) #expanded SS in matrix form
X.SS.vec<-reduc.ext.X.SS[[2]] #expanded SS in vector form

#What the current progress of runs in the exisiting states
reduc.ext_state_prog<-FINAL.FMCI_SS.current.prog(r.tuples=reduc.ext.X.SS[[2]], r.tuples.mat=reduc.ext.X.SS[[1]], symb=base.X.SS, k=order.m)
reduc.ext_state_prog<-as.numeric(matrix(unlist(strsplit(reduc.ext_state_prog, "\\|")), ncol=2, byrow=T)[,2]) #in nice numerical form

state.int.prob<-which(reduc.ext_state_prog>=order.m) #Those which satisfy the desired sustained period and thus will have interesting transition probabilities
state.boring.prob<-which(reduc.ext_state_prog<order.m) #And those which have a deterministic set up

new.no.X<-length(X.SS.vec)
new.TP.mat<-matrix(0, nrow= length(X.SS.vec), ncol= length(base.X.SS))
new.TP.mat[cbind(state.boring.prob, as.numeric(X.SS.mat[state.boring.prob, order.m]))]<-1 #The boring state transition mechanism is deterministic

new.TP.mat[state.int.prob, ]<- base.X.TP.mat[as.numeric(X.SS.mat[state.int.prob,order.m]), ] #The interesting state transitions
dimnames(new.TP.mat)<-list(X.SS.vec, base.X.SS)
new.TP.mat #The TP matrix with respect to the expanded SS
}

###########
reduc.square.TP.mat<-function(rect.TP.mat, reduc.X.SS.info){
#Function to convert a rectangle TP matrix defined over the reduced SS considered.
	#To a nice square matrix going from m-tuples to m-tuples

#rect.TP.mat=the rectangular m-th order CP
#reduc.X.SS.info=the new expanded, reduced X SS

reduc.X.SS<-rownames(rect.TP.mat) #The reduced SS considered
base.X.SS<-colnames(rect.TP.mat) #The base SS 

no.red.Xsize<-length(reduc.X.SS) #Size of the reduced SS
#temp.sq.TP.mat<-matrix(0, nrow=no.red.Xsize, ncol=no.red.Xsize)
#dimnames(temp.sq.TP.mat)<-list(reduc.X.SS, reduc.X.SS)

#Only considering legitimate moves
non.zero.TP.mat<-rect.TP.mat!=0 #T/F to highlight which are the non-zero probabilities and thus the legitimate moves
non.zero.row<-row(rect.TP.mat)[non.zero.TP.mat] #Row locations of the non-zero probabilities
non.zero.col<-col(rect.TP.mat)[non.zero.TP.mat] #Column locations of the non-zero proababilities
non.zero.prob.vec<-rect.TP.mat[non.zero.TP.mat] #The correponding non-zero probabilities

reduc.X.SS.mat<-as.matrix(reduc.X.SS.info[[1]]) #Matrix representation of the state space
order.m<-ncol(reduc.X.SS.mat)
destin.state.mat<-reduc.X.SS.mat[non.zero.row, 2:order.m, drop=F] #Continuing the previous part of the tuple for states of interest
destin.base.state<-base.X.SS[non.zero.col]#And where they go to for time t
destin.state.mat<-cbind(destin.state.mat, destin.base.state) #The complete product
destin.state.vec<-apply(destin.state.mat, FUN=paste, 1, collapse=",")

#Where is this in the actual statespace of interest
destin.col.coord<-match(destin.state.vec, reduc.X.SS) #This might also come in useful for FMCI stuff

#temp.sq.TP.mat[cbind(non.zero.row, destin.col.coord)]<-non.zero.prob.vec

#If we're being very fancy, then Sparse matrices might come in useful here...for later?
temp.sq.TP.mat<-sparseMatrix(i=non.zero.row, j=destin.col.coord, x=non.zero.prob.vec, dims=c(no.red.Xsize, no.red.Xsize), dimnames=list(reduc.X.SS, reduc.X.SS))
}

############
base.to.reduc.TP.mat<-function(base.TP.mat, order.m){
#Complete function going from base state space and it's transition matrix, to m-tuple reduced SS and it's corresponding square transition matrix

#base.TP.mat=original statespace
#order.m= order of the HMM


base.X.SS<-colnames(base.TP.mat)

#Form the reduced SS
reduc.ext.SS_info<-reduc.ext.X.SS(base.X.SS=base.X.SS, order.m=order.m)

#Form the rectangular TP matrix
rectangle.TP<-reduc.ext.X.SS.prob(base.X.SS=base.X.SS, order.m=order.m, reduc.ext.X.SS=reduc.ext.SS_info, base.X.TP.mat=base.TP.mat)

#Form the equivalend square TP matrix
reduc.sq.TP.mat<-reduc.square.TP.mat(rect.TP.mat=rectangle.TP, reduc.X.SS.info=reduc.ext.SS_info)
list("Reduced Sq TP matrix"=reduc.sq.TP.mat, "R.X_SS.mat"=reduc.ext.SS_info[[1]], "R.X_SS.vec"=reduc.ext.SS_info[[2]])
}



#################
#Functions concerning transforming a first order Markov Transition matrix
	#into a r-th order Markov Transition Matrix

#Doesn't really have a purpose, just helps me occasionally when testing.

########

tup.to.rtup<-function(TP.mat, r.val){
#Convert a transition matrix for a first order MC into a transition matrix involving r-tuples 
    #This for the case when the underlying MC follows a first order MC
    #But there is the observations are dependent on the previous r observation
    #This thus induces an rth order MC

#TP.mat=TP matrix
#r.val=order we would like to induce

#r.val=length tuples we are interested in coverting to
no.X<-nrow(TP.mat)
X.statespace<-c(1:no.X)

repeat.SS<-rep(list(X.statespace), r.val+1) #List containing the statespace repeat r.val+1
rp1.tuples.mat<-expand.grid(repeat.SS) #All possible r.val+1 tuples
r.tuples.SS<-unique(apply(rp1.tuples.mat[,1:r.val, drop=F], 1, FUN=paste, collapse=",")) # r-tuples for the new statespace

#Now to place the exisitng transition probabilities into their new place
locations.info<-tup.to.rtup.destin(rp1.tuples.mat=rp1.tuples.mat,r.tuples.SS=r.tuples.SS, no.X=no.X)

extended.SS<-as.matrix(locations.info[[1]])
orig.SS<-as.matrix(locations.info[[2]])

extend.TP.mat<-matrix(0, nrow=length(r.tuples.SS), ncol=length(r.tuples.SS))
extend.TP.mat[extended.SS]<-TP.mat[orig.SS]
dimnames(extend.TP.mat)<-list(r.tuples.SS, r.tuples.SS)
return(extend.TP.mat) #TP matrix on expanded SS
}
##############
tup.to.rtup.sparse<-function(TP.mat, r.val){
#Convert a transition matrix for a first order MC into a transition matrix involving r-tuples 
    #This for the case when the underlying MC follows a first order MC
    #But there is the observations are dependent on the previous r observation
    #This thus induces an rth order MC
#Sparse version

#r.val=length tuples we are interested in coverting to
no.X<-nrow(TP.mat)
X.statespace<-c(1:no.X)

repeat.SS<-rep(list(X.statespace), r.val+1) #List containing the statespace repeat r.val+1
rp1.tuples.mat<-expand.grid(repeat.SS) #All possible r.val+1 tuples
r.tuples.SS<-unique(apply(rp1.tuples.mat[,1:r.val, drop=F], 1, FUN=paste, collapse=",")) # r-tuples for the new statespace

#Now to place the exisitng transition probabilities into their new place
locations.info<-tup.to.rtup.destin(rp1.tuples.mat=rp1.tuples.mat,r.tuples.SS=r.tuples.SS, no.X=no.X)

extended.SS<-as.matrix(locations.info[[1]])
orig.SS<-as.matrix(locations.info[[2]])

#extend.TP.mat<-matrix(0, nrow=length(r.tuples.SS), ncol=length(r.tuples.SS))
#extend.TP.mat[extended.SS]<-TP.mat[orig.SS]
tp.prob.vec<-TP.mat[orig.SS]
extend.TP.mat<-sparseMatrix(i=extended.SS[,1], j=extended.SS[,2], x=tp.prob.vec, dimnames=list(r.tuples.SS, r.tuples.SS))
dimnames(extend.TP.mat)<-list(r.tuples.SS, r.tuples.SS)
return(extend.TP.mat) #The TP matrix defined on the expanded SS
}

##############

tup.to.rtup.destin<-function(rp1.tuples.mat,r.tuples.SS, no.X){
#Function to determine the destinations of where to place the new co-ordinates

rval<-ncol(rp1.tuples.mat)-1
start.pos<-rep(1:no.X, ncol(rp1.tuples.mat)-1)

#Start and end in terms of r-tuples
start.tuples.vec<-apply(rp1.tuples.mat[,-(rval+1), drop=F], 1, FUN=paste, collapse=",")
destin.tuples.vec<-apply(rp1.tuples.mat[,-1, drop=F], 1, FUN=paste, collapse=",")

#Now in terms of locations within the extended statespace
start.loc.vec<-match(start.tuples.vec, r.tuples.SS)
end.loc.vec<-match(destin.tuples.vec, r.tuples.SS)

#Now in matrix form
new.coord<-cbind(start.loc.vec, end.loc.vec)

#Which co-ordinates to look at in the original transition matrix
orig.coord<-rp1.tuples.mat[,rval:(rval+1), drop=F]

list("New placement locations"=new.coord, "Exisitng locations"=orig.coord)
}


#################
rtup.TP.mat.square.func<-function(rtup.TP.mat){
#Function to convert a  r-tuple transition matrix into a square one
X.SS<-colnames(rtup.TP.mat); no.X<-length(X.SS)
X.rtuple.SS<-rownames(rtup.TP.mat); no.X.rtuple<-length(X.rtuple.SS)
rval<-log(no.X.rtuple, base=no.X)

#Starting postions and destinations
rp1.tuple.mat<-as.matrix(expand.grid(rep(list(X.SS), rval+1)))
start.rtuple<-apply(rp1.tuple.mat[,-(rval+1), drop=F], 1, FUN=paste, collapse=","); start.index<-match(start.rtuple, X.rtuple.SS)
destin.rtuple<-apply(rp1.tuple.mat[,-1, drop=F], 1, FUN=paste, collapse=","); destin.index<-match(destin.rtuple, X.rtuple.SS)

#Placement of transition probabilties
square.temp.mat<-matrix(0, nrow=no.X.rtuple, ncol=no.X.rtuple)
dimnames(square.temp.mat)<-list(X.rtuple.SS, X.rtuple.SS)
square.temp.mat[cbind(start.rtuple, destin.rtuple)]<-rtup.TP.mat[cbind(start.rtuple, rp1.tuple.mat[, rval+1, drop=F])]
square.temp.mat
}

################
FINAL.FMCI_SS.current.prog<-function (r.tuples, r.tuples.mat, symb.vec, k) 
{
#This function figures out progress in tuples
    r <- ncol(r.tuples.mat)
    curr.pattern.prog <- rep(0, nrow(r.tuples.mat))
    no.symb <- length(symb.vec)
    for (i in 1:min(k, r)) {
        reduc.rtuples <- r.tuples.mat[, (r - i + 1):r, drop = F]
        reduc.rtuples <- apply(reduc.rtuples, 1, FUN = paste, 
            collapse = ",")
        auxiliary.match <- matrix(symb.vec, ncol = i, nrow = no.symb, 
            byrow = F)
        auxiliary.match <- apply(auxiliary.match, 1, FUN = paste, 
            collapse = ",")
        locat <- which(match(reduc.rtuples, auxiliary.match, 
            nomatch = 0) != 0)
        curr.pattern.prog[locat] <- i
    }
    curr.states <- paste(r.tuples, curr.pattern.prog, sep = "|")
    curr.ext.states <- NULL
    curr.states
}
