#Supplementary functions
#Extract desired number of scaled processes when supplied with a Wavelet Periodogram

WavPer.scale.proc.mat<-function(WavPer, J.star=2, data.J=NULL){
#Function to provide the desired matrix form of the wavelet periodogram
#WavPer = wavelet periodogram (can be smoothed/corrected/raw)
#J.star = the number of scale processes we want to consider
#data.J=total number of dyadic scales of time series

if(is.null(data.J)==TRUE){data.J<-length(WavPer)}#+1 #The number of dyadic scales
data.T<-2^(data.J) #This tells us how many observations are in each scale process

I.mat<-matrix(0, ncol=data.T, nrow=J.star) #Storage of wavelet periodogram

for(scale in 1:J.star){
scale.proc<-accessD(WavPer, level=data.J-scale) #The coefficients for the particular scale
I.mat[scale,]<-jitter(scale.proc) #Populate the storage matrix. Jitter to avoid wavelet coefficient being 0 which would otherwise affect the density calculation 
}
#Each row corresponds to a different scale (getting coarser)
	#Each column represents a different time location
I.mat #Wavelet periodogram in matrix form
}

######
extW2.mat.construct<-function(mtuple.vec, W2.mat){
#Function to construct the corresponding W2.ext.mat which is dependent on the underlying state configuration at that time.
#mtuple.vec=the state vector which W2.mat will be based upon
#W2.mat = W2 for a particularly state and scale process
	#Each row=state
	#Each column= scale process
#Furthest RHS of mtuple.vec corresponds to the most recent time point

extW2.mat<-W2.mat[mtuple.vec,]
extW2.mat<-t(extW2.mat)
	#So that each row = scale process
	#Each column = time point, RHS=most recent
extW2.mat
}

#####
sqMVN.data.density<-function(sq.data.vec, SigmaD.mat){
#Function to compute the density of squared MVN squared data
#The density is based on a change of variables argument

#sq.data.vec=vector of time series points across all scales considered at a single time point.
#SigmaD.mat=the state dependent covariance matrix.

no.proc<-length(sq.data.vec) #The number of scale processes we are considering
#Alternating sign matrix
alt.sign.label<-paste("Alt", "Sign", "Mat", no.proc, sep=".")
if(exists(alt.sign.label)) {alt.sign.mat<-get(alt.sign.label)}
else{alt.sign.mat<-as.matrix(expand.grid(rep(list(c(-1, 1)), no.proc)))
assign(alt.sign.label, alt.sign.mat)
}

no.combns<-nrow(alt.sign.mat)
data.vec<-sqrt(sq.data.vec) #The squared version of the data that we need
data.mat.rep<-matrix(data.vec, ncol=no.proc, nrow=no.combns, byrow=T)
sign.data.mat<-data.mat.rep*alt.sign.mat
MVN.part.vec<-sum(apply(sign.data.mat, 1, FUN=dmvnorm, sigma=SigmaD.mat)) #The MVN part
denominator<-(2^no.proc)*prod(data.vec) #The change of variables part
MVN.part.vec/denominator #Corresponding density
}

#######
discrete.wavelets<-function (J, filter.number, filter.family) {
#Function to compute G filter coefficients
    dwname <- paste("dw.", J, ".", filter.number, ".", filter.family, 
        sep = "")
#    if (exists(dwname)) 
#        return(get(dwname))
    longest.support <- (2^J - 1) * (2 * filter.number - 1) + 
        1
    J1 <- ceiling(logb(longest.support, 2))
    tmp <- rep(0, max(2^J1, 4))
    tmp.w <- wd(tmp, filter.number, filter.family) #DWT object...
    dw <- vector("list", J) #Object to store the G filtering coefficients
    index <- filter.number
    for (j in 1:J) {
        tmp.w$D[index] <- 1
        dw[[j]] <- wr(tmp.w)[1:((2^j - 1) * (2 * filter.number - 
            1) + 1)]
        tmp.w$D[index] <- 0
        index <- index + 2^(J1 - j)
    }
    assign(dwname, dw, pos = 1)
    return(dw)
}
	
