setwd("/storage/striaj/LSW_HMM_Paper_Code/Functions_Folder/") #Change directory as necesary

#Necessary packages 
library(wavethresh)
library(Matrix)
library(mvtnorm)
library(compiler)
enableJIT(3) #Apparently this will speed up some things in R

#Source files
source("ForwBack_Dep_WavPer.R") #Functions associated with log-lik and Forward-Backwards equations under the LSW-HMM framework

source("SMC_Dep_WavPer.R") #The main SMC samplers function for the LSW-HMM framework
	source("SMC_auxiliary.r") #Supplementary SMC functions which are not specific to the LSW-HMM framework (e.g. TP proposals, resampling

source("SMC_Dep_WavPer_CP.R") #The CP function which utilises the SMC samplers output
	source("FMCI_SS_kstar_ReducSS.r") #Functions associated with FMCI state-space computation
	source("Reduced-mtuple_SS.R") #Functions associated withW FMCI state-space computation
	source("FMCI_SS_Supple.R") #Supplementary functions regarding FMCI
	source("Cver_Cp_open_func_Sparse.r")
	source("Cver_CP_via_FMCI_R_Sparse.r")
	source("Cver_UpdChainRow_R_Sparse.r")

source("Supplementary_Functions.r") #Supplementary functions associated with HMM SS
source("Wavelet_Imat_construc.r") #Wavelet analysis supplementary functions e.g. extracting wavelet periodogram analysis

#Simulate some white noise data
sd.vec<-c(1,2,1,2)
New.data<-c(rnorm(150, 0, sd.vec[1]), rnorm(150, 0, sd.vec[2]), rnorm(150, 0, sd.vec[3]), rnorm(62, 0, sd.vec[4]))
data.T<-length(New.data)

CP.loc.vec<-c(151,301, 451) #Locations of changepoints in the simulated data

#Now obtain the raw wavelet periodogram
a.fno<-1; a.f.fam<-"DaubExPhase" #Associated with Haar mother wavelet
Raw.WavPer<-ewspec(New.data, family=a.f.fam, filter.number=a.fno)$Wav

no.proc<-2 #Number of scale processes we are going to consider. This is J^star in the paper
I.mat<-WavPer.scale.proc.mat(WavPer=Raw.WavPer, J.star=no.proc, data.J=log2(data.T)) #To obtain the data matrix that we're going to analyse

#SMC paramters
N<-5; B<-5 #Number of particles and number of distribution
a.state<-2 #The number of states assumed in our analysis

hyper.vec<-c(10,1,1) #Hyperparameters, 1st concerns TP matrices, 2nd and 3rd concerns inverse powers

prop.var.vec<-c(10,10) #Proposal variances, 1st concerns the TP, 2nd concerns the inverse powers

#SMC info
SMC_info<-Multi.Dep_WavPer.SMC_complete(data.mat=I.mat, no.X=a.state, init.vec=c(1, rep(0, a.state-1)), N=N, B=B, temp.sched=seq(0,1, len=B), hyper.vec=hyper.vec, prop.var.vec=prop.var.vec, filter.number=a.fno, filter.family=a.f.fam)
#Takes a very long time!


#Some orderinng the particles so we can perform some valid (changepoint inference)
ordered.theta.list<-SMC_info[[1]]

if(a.state!=1){
for(part in 1:N){
theta<-ordered.theta.list[[part]]

TP.mat<-theta[[1]]; print(TP.mat)
power.mat<-theta[[2]]; print(power.mat)
print(theta)
ordering<-order(power.mat[,1, drop=F], decreasing=TRUE) 


TP.mat<-TP.mat[ordering, ordering, drop=F]
	dimnames(TP.mat)<-list(1:a.state, 1:a.state)
power.mat<-power.mat[ordering,, drop=F]

theta[[1]]<-TP.mat
theta[[2]]<-power.mat

ordered.theta.list[[part]]<-theta
}}

#CP run lengths
krun<-20 #Required sustained change in state in underlying statesequence for a changepoint into a regime to have occurred
k.star<-10 #Required break in underlying statesequence for changepoint out of a regime to have occurred



#Wavelet info, namely the wavelet filtering coefficients
filter.info<-discrete.wavelets(filter.number=a.fno, filter.family=a.f.fam, J=no.proc)

#The main CP function
CP_info<-SMC_CPviaFMCI_Multi.Dep_WavPer(SMC_particles=ordered.theta.list, SMC_weights=SMC_info[[2]], krun=krun, k.star=k.star, data.mat=I.mat, X.init.vec=c(1, rep(0, a.state-1)), filter.info=filter.info)
 
#Producing some plots


iCPPmat<-CP_info[[1]]
CPP.vec<-rowSums(iCPPmat)
plot(CPP.vec, ylab="Changepoint Probability", type="l")
abline(v=CP.loc.vec, lty=2, col="red")
#Probability of a changepoint occuring at each time point

DistCP<-CP_info[[3]]
DistCP<-c(1-sum(DistCP), DistCP) #To have the probability of 0 CPs as well
max.distCP<-10 #Max number of CPs considered in plotting the distribution of number of CPs
barplot(DistCP[1:(max.distCP+1)], names.arg=0:max.distCP, xlab="No. CPs", ylab="Probability", main="Distribution of Number of CPs")

save.image("WN_WS.RData")
